@@ -32,6 +32,7 @@
 #include <string.h>
 #include <sys/mman.h>
 #include <sys/signalfd.h>
+#include <sys/types.h>
 #include <unistd.h>
 #include <wayland-client-core.h>
 #include <wayland-client-protocol.h>
@@ -40,6 +41,7 @@
 #include <xkbcommon/xkbcommon.h>
 
 #include "wlmaker-icon-unstable-v1-client-protocol.h"
+#include "ext-input-observation-v1-client-protocol.h"
 #include "xdg-shell-client-protocol.h"
 #include "xdg-decoration-client-protocol.h"
 
@@ -278,6 +280,8 @@ static const object_t objects[] = {
       offsetof(wlclient_attributes_t, icon_manager_ptr), NULL },
     { &zxdg_decoration_manager_v1_interface, 1,
       offsetof(wlclient_attributes_t, xdg_decoration_manager_ptr), NULL },
+    { &ext_input_observation_manager_v1_interface, 1,
+      offsetof(wlclient_attributes_t, input_observation_manager_ptr), NULL },
     { NULL, 0, 0, NULL }  // sentinel.
 
 };
@@ -379,6 +383,12 @@ wlclient_t *wlclient_create(const char *app_id_ptr)
         return NULL;
     }
 
+    // Hack: Somehow this propagates the protool far enough for getting
+    // the pointer registered.
+    wl_display_roundtrip(wlclient_ptr->attributes.wl_display_ptr);
+    wl_display_roundtrip(wlclient_ptr->attributes.wl_display_ptr);
+    wl_display_roundtrip(wlclient_ptr->attributes.wl_display_ptr);
+
     return wlclient_ptr;
 }
 
@@ -730,15 +740,17 @@ void wlc_seat_handle_capabilities(
     wlclient_t *client_ptr = data_ptr;
 
     bool supports_pointer = capabilities & WL_SEAT_CAPABILITY_POINTER;
-    if (supports_pointer && NULL == client_ptr->wl_pointer_ptr) {
-        client_ptr->wl_pointer_ptr = wl_seat_get_pointer(wl_seat_ptr);
+    if (supports_pointer && NULL == client_ptr->attributes.wl_pointer_ptr) {
+        client_ptr->attributes.wl_pointer_ptr =
+            wl_seat_get_pointer(wl_seat_ptr);
         wl_pointer_add_listener(
-            client_ptr->wl_pointer_ptr,
+            client_ptr->attributes.wl_pointer_ptr,
             &wlc_pointer_listener,
             client_ptr);
-    } else if (!supports_pointer && NULL != client_ptr->wl_pointer_ptr) {
-        wl_pointer_release(client_ptr->wl_pointer_ptr);
-        client_ptr->wl_pointer_ptr = NULL;
+    } else if (!supports_pointer &&
+               NULL != client_ptr->attributes.wl_pointer_ptr) {
+        wl_pointer_release(client_ptr->attributes.wl_pointer_ptr);
+        client_ptr->attributes.wl_pointer_ptr = NULL;
     }
 
     bool supports_keyboard = capabilities & WL_SEAT_CAPABILITY_KEYBOARD;
