@@ -20,20 +20,31 @@
 
 #include "action.h"
 
-#include "default_configuration.h"
-#include "root_menu.h"
-#include "server.h"
-#include "conf/decode.h"
-#include "conf/plist.h"
-
+#include <libbase/libbase.h>
+#include <libbase/plist.h>
+#include <stdbool.h>
+#include <stdint.h>
+#include <stdlib.h>
+#include <string.h>
+#include <wayland-server-core.h>
+#include <xkbcommon/xkbcommon-keysyms.h>
 #include <xkbcommon/xkbcommon.h>
-
 #define WLR_USE_UNSTABLE
-#include <wlr/backend/session.h>
 #include <wlr/types/wlr_cursor.h>
 #include <wlr/types/wlr_keyboard.h>
 #undef WLR_USE_UNSTABLE
 
+#include "backend/backend.h"
+#include "background.h"
+#include "cursor.h"
+#include "default_configuration.h"
+#include "idle.h"
+#include "keyboard.h"
+#include "root_menu.h"
+#include "server.h"
+#include "subprocess_monitor.h"
+#include "toolkit/toolkit.h"
+
 /* == Declarations ========================================================= */
 
 /** State of the bound actions. */
@@ -42,6 +53,8 @@ struct _wlmaker_action_handle_t {
     bs_dequeue_t              bindings;
     /** Back-link to server state. */
     wlmaker_server_t          *server_ptr;
+    /** Whether to add 'Logo' to the bindings. */
+    bool                      add_logo;
 };
 
 /** Key binding for a standard action. */
@@ -65,14 +78,11 @@ static bool _wlmaker_keybindings_parse(
 
 static bool _wlmaker_keybindings_bind_item(
     const char *key_ptr,
-    wlmcfg_object_t *object_ptr,
+    bspl_object_t *object_ptr,
     void *userdata_ptr);
 
 static bool _wlmaker_action_bound_callback(
     const wlmaker_key_combo_t *binding_ptr);
-static void _wlmaker_action_switch_to_vt(
-    wlmaker_server_t *server_ptr,
-    unsigned vt_num);
 
 /* == Data ================================================================= */
 
@@ -80,61 +90,76 @@ static void _wlmaker_action_switch_to_vt(
 const char *wlmaker_action_config_dict_key = "KeyBindings";
 
 /** Supported modifiers for key bindings. */
-static const wlmcfg_enum_desc_t _wlmaker_keybindings_modifiers[] = {
-    WLMCFG_ENUM("Shift", WLR_MODIFIER_SHIFT),
-    // Caps? Maybe not: WLMCFG_ENUM("Caps", WLR_MODIFIER_CAPS),
-    WLMCFG_ENUM("Ctrl", WLR_MODIFIER_CTRL),
-    WLMCFG_ENUM("Alt", WLR_MODIFIER_ALT),
-    WLMCFG_ENUM("Mod2", WLR_MODIFIER_MOD2),
-    WLMCFG_ENUM("Mod3", WLR_MODIFIER_MOD3),
-    WLMCFG_ENUM("Logo", WLR_MODIFIER_LOGO),
-    WLMCFG_ENUM("Mod5", WLR_MODIFIER_MOD5),
-    WLMCFG_ENUM_SENTINEL(),
+static const bspl_enum_desc_t _wlmaker_keybindings_modifiers[] = {
+    BSPL_ENUM("Shift", WLR_MODIFIER_SHIFT),
+    // Caps? Maybe not: BSPL_ENUM("Caps", WLR_MODIFIER_CAPS),
+    BSPL_ENUM("Ctrl", WLR_MODIFIER_CTRL),
+    BSPL_ENUM("Alt", WLR_MODIFIER_ALT),
+    BSPL_ENUM("Mod2", WLR_MODIFIER_MOD2),
+    BSPL_ENUM("Mod3", WLR_MODIFIER_MOD3),
+    BSPL_ENUM("Logo", WLR_MODIFIER_LOGO),
+    BSPL_ENUM("Mod5", WLR_MODIFIER_MOD5),
+    BSPL_ENUM_SENTINEL(),
 };
 
 /** The actions that can be bound. */
-const wlmcfg_enum_desc_t wlmaker_action_desc[] = {
-    WLMCFG_ENUM("None", WLMAKER_ACTION_NONE),
-    WLMCFG_ENUM("Quit", WLMAKER_ACTION_QUIT),
-    WLMCFG_ENUM("LockScreen", WLMAKER_ACTION_LOCK_SCREEN),
-    WLMCFG_ENUM("InhibitLockBegin", WLMAKER_ACTION_LOCK_INHIBIT_BEGIN),
-    WLMCFG_ENUM("InhibitLockEnd", WLMAKER_ACTION_LOCK_INHIBIT_END),
-    WLMCFG_ENUM("LaunchTerminal", WLMAKER_ACTION_LAUNCH_TERMINAL),
-    WLMCFG_ENUM("ShellExecute", WLMAKER_ACTION_SHELL_EXECUTE),
+const bspl_enum_desc_t wlmaker_action_desc[] = {
+    BSPL_ENUM("None", WLMAKER_ACTION_NONE),
+    BSPL_ENUM("Quit", WLMAKER_ACTION_QUIT),
+    BSPL_ENUM("LockScreen", WLMAKER_ACTION_LOCK_SCREEN),
+    BSPL_ENUM("InhibitLockBegin", WLMAKER_ACTION_LOCK_INHIBIT_BEGIN),
+    BSPL_ENUM("InhibitLockEnd", WLMAKER_ACTION_LOCK_INHIBIT_END),
+    BSPL_ENUM("LaunchTerminal", WLMAKER_ACTION_LAUNCH_TERMINAL),
+    BSPL_ENUM("ShellExecute", WLMAKER_ACTION_SHELL_EXECUTE),
+    BSPL_ENUM("Execute", WLMAKER_ACTION_EXECUTE),
 
-    WLMCFG_ENUM("WorkspacePrevious", WLMAKER_ACTION_WORKSPACE_TO_PREVIOUS),
-    WLMCFG_ENUM("WorkspaceNext", WLMAKER_ACTION_WORKSPACE_TO_NEXT),
+    BSPL_ENUM("WorkspacePrevious", WLMAKER_ACTION_WORKSPACE_TO_PREVIOUS),
+    BSPL_ENUM("WorkspaceNext", WLMAKER_ACTION_WORKSPACE_TO_NEXT),
+    BSPL_ENUM("WorkspaceAdd", WLMAKER_ACTION_WORKSPACE_ADD),
+    BSPL_ENUM("WorkspaceDestroyLast", WLMAKER_ACTION_WORKSPACE_DESTROY_LAST),
 
-    WLMCFG_ENUM("TaskPrevious", WLMAKER_ACTION_TASK_TO_PREVIOUS),
-    WLMCFG_ENUM("TaskNext", WLMAKER_ACTION_TASK_TO_NEXT),
+    BSPL_ENUM("TaskPrevious", WLMAKER_ACTION_TASK_TO_PREVIOUS),
+    BSPL_ENUM("TaskNext", WLMAKER_ACTION_TASK_TO_NEXT),
 
-    WLMCFG_ENUM("WindowRaise", WLMAKER_ACTION_WINDOW_RAISE),
-    WLMCFG_ENUM("WindowLower", WLMAKER_ACTION_WINDOW_LOWER),
-    WLMCFG_ENUM("WindowToggleFullscreen", WLMAKER_ACTION_WINDOW_TOGGLE_FULLSCREEN),
-    WLMCFG_ENUM("WindowToggleMaximized", WLMAKER_ACTION_WINDOW_TOGGLE_MAXIMIZED),
+    BSPL_ENUM("WindowRaise", WLMAKER_ACTION_WINDOW_RAISE),
+    BSPL_ENUM("WindowLower", WLMAKER_ACTION_WINDOW_LOWER),
+    BSPL_ENUM("WindowToggleFullscreen", WLMAKER_ACTION_WINDOW_TOGGLE_FULLSCREEN),
+    BSPL_ENUM("WindowToggleMaximized", WLMAKER_ACTION_WINDOW_TOGGLE_MAXIMIZED),
 
-    WLMCFG_ENUM("WindowMaximize", WLMAKER_ACTION_WINDOW_MAXIMIZE),
-    WLMCFG_ENUM("WindowUnmaximize", WLMAKER_ACTION_WINDOW_UNMAXIMIZE),
-    WLMCFG_ENUM("WindowFullscreen", WLMAKER_ACTION_WINDOW_FULLSCREEN),
-    WLMCFG_ENUM("WindowShade", WLMAKER_ACTION_WINDOW_SHADE),
-    WLMCFG_ENUM("WindowUnshade", WLMAKER_ACTION_WINDOW_UNSHADE),
+    BSPL_ENUM("WindowMaximize", WLMAKER_ACTION_WINDOW_MAXIMIZE),
+    BSPL_ENUM("WindowUnmaximize", WLMAKER_ACTION_WINDOW_UNMAXIMIZE),
+    BSPL_ENUM("WindowFullscreen", WLMAKER_ACTION_WINDOW_FULLSCREEN),
+    BSPL_ENUM("WindowShade", WLMAKER_ACTION_WINDOW_SHADE),
+    BSPL_ENUM("WindowUnshade", WLMAKER_ACTION_WINDOW_UNSHADE),
+    BSPL_ENUM("WindowClose", WLMAKER_ACTION_WINDOW_CLOSE),
+    BSPL_ENUM("WindowToNextWorkspace", WLMAKER_ACTION_WINDOW_TO_NEXT_WORKSPACE),
+    BSPL_ENUM("WindowToPreviousWorkspace", WLMAKER_ACTION_WINDOW_TO_PREVIOUS_WORKSPACE),
 
-    WLMCFG_ENUM("RootMenu", WLMAKER_ACTION_ROOT_MENU),
+    BSPL_ENUM("RootMenu", WLMAKER_ACTION_ROOT_MENU),
 
-    WLMCFG_ENUM("SwitchToVT1", WLMAKER_ACTION_SWITCH_TO_VT1),
-    WLMCFG_ENUM("SwitchToVT2", WLMAKER_ACTION_SWITCH_TO_VT2),
-    WLMCFG_ENUM("SwitchToVT3", WLMAKER_ACTION_SWITCH_TO_VT3),
-    WLMCFG_ENUM("SwitchToVT4", WLMAKER_ACTION_SWITCH_TO_VT4),
-    WLMCFG_ENUM("SwitchToVT5", WLMAKER_ACTION_SWITCH_TO_VT5),
-    WLMCFG_ENUM("SwitchToVT6", WLMAKER_ACTION_SWITCH_TO_VT6),
-    WLMCFG_ENUM("SwitchToVT7", WLMAKER_ACTION_SWITCH_TO_VT7),
-    WLMCFG_ENUM("SwitchToVT8", WLMAKER_ACTION_SWITCH_TO_VT8),
-    WLMCFG_ENUM("SwitchToVT9", WLMAKER_ACTION_SWITCH_TO_VT9),
-    WLMCFG_ENUM("SwitchToVT10", WLMAKER_ACTION_SWITCH_TO_VT10),
-    WLMCFG_ENUM("SwitchToVT11", WLMAKER_ACTION_SWITCH_TO_VT11),
-    WLMCFG_ENUM("SwitchToVT12", WLMAKER_ACTION_SWITCH_TO_VT12),
+    BSPL_ENUM("OutputMagnify", WLMAKER_ACTION_OUTPUT_MAGNIFY),
+    BSPL_ENUM("OutputReduce", WLMAKER_ACTION_OUTPUT_REDUCE),
+    BSPL_ENUM("OutputSaveState", WLMAKER_ACTION_OUTPUT_SAVE_STATE),
 
-    WLMCFG_ENUM_SENTINEL(),
+    BSPL_ENUM("SwitchToVT1", WLMAKER_ACTION_SWITCH_TO_VT1),
+    BSPL_ENUM("SwitchToVT2", WLMAKER_ACTION_SWITCH_TO_VT2),
+    BSPL_ENUM("SwitchToVT3", WLMAKER_ACTION_SWITCH_TO_VT3),
+    BSPL_ENUM("SwitchToVT4", WLMAKER_ACTION_SWITCH_TO_VT4),
+    BSPL_ENUM("SwitchToVT5", WLMAKER_ACTION_SWITCH_TO_VT5),
+    BSPL_ENUM("SwitchToVT6", WLMAKER_ACTION_SWITCH_TO_VT6),
+    BSPL_ENUM("SwitchToVT7", WLMAKER_ACTION_SWITCH_TO_VT7),
+    BSPL_ENUM("SwitchToVT8", WLMAKER_ACTION_SWITCH_TO_VT8),
+    BSPL_ENUM("SwitchToVT9", WLMAKER_ACTION_SWITCH_TO_VT9),
+    BSPL_ENUM("SwitchToVT10", WLMAKER_ACTION_SWITCH_TO_VT10),
+    BSPL_ENUM("SwitchToVT11", WLMAKER_ACTION_SWITCH_TO_VT11),
+    BSPL_ENUM("SwitchToVT12", WLMAKER_ACTION_SWITCH_TO_VT12),
+
+    // A duplicate to ShellExecute, permits `wmmenugen` compatibility.
+    BSPL_ENUM("SHEXEC", WLMAKER_ACTION_SHELL_EXECUTE),
+    // A duplicate to Execute, permits compatibility with Window Maker.
+    BSPL_ENUM("EXEC", WLMAKER_ACTION_EXECUTE),
+
+    BSPL_ENUM_SENTINEL(),
 };
 
 /* == Exported methods ===================================================== */
@@ -142,14 +167,16 @@ const wlmcfg_enum_desc_t wlmaker_action_desc[] = {
 /* ------------------------------------------------------------------------- */
 wlmaker_action_handle_t *wlmaker_action_bind_keys(
     wlmaker_server_t *server_ptr,
-    wlmcfg_dict_t *keybindings_dict_ptr)
+    bspl_dict_t *keybindings_dict_ptr,
+    bool add_logo)
 {
     wlmaker_action_handle_t *handle_ptr = logged_calloc(
         1, sizeof(wlmaker_action_handle_t));
     if (NULL == handle_ptr) return NULL;
     handle_ptr->server_ptr = server_ptr;
+    handle_ptr->add_logo = add_logo;
 
-    if (wlmcfg_dict_foreach(
+    if (bspl_dict_foreach(
             keybindings_dict_ptr,
             _wlmaker_keybindings_bind_item,
             handle_ptr)) {
@@ -184,6 +211,7 @@ void wlmaker_action_execute(wlmaker_server_t *server_ptr,
 {
     wlmtk_workspace_t *workspace_ptr, *next_workspace_ptr;
     wlmtk_window_t *window_ptr;
+    const char **argv;
 
     switch (action) {
     case WLMAKER_ACTION_NONE:
@@ -208,17 +236,23 @@ void wlmaker_action_execute(wlmaker_server_t *server_ptr,
         break;
 
     case WLMAKER_ACTION_LAUNCH_TERMINAL:
-        if (0 == fork()) {
-            execl("/bin/sh", "/bin/sh", "-c", "/usr/bin/foot", (void *)NULL);
-        }
+        argv = (const char*[]){ "/bin/sh", "-c", "/usr/bin/foot", NULL };
+        wlmaker_subprocess_monitor_run(
+            server_ptr->monitor_ptr,
+            bs_subprocess_create(argv[0], argv, NULL));
         break;
 
     case WLMAKER_ACTION_SHELL_EXECUTE:
-        if (NULL == arg_ptr) {
-            bs_log(BS_ERROR, "Invalid argument NULL for 'Execute'.");
-        } else if (0 == fork()) {
-            execl("/bin/sh", "/bin/sh", "-c", arg_ptr, (void *)NULL);
-        }
+        argv = (const char*[]){ "/bin/sh", "-c", arg_ptr, NULL };
+        wlmaker_subprocess_monitor_run(
+            server_ptr->monitor_ptr,
+            bs_subprocess_create(argv[0], argv, NULL));
+        break;
+
+    case WLMAKER_ACTION_EXECUTE:
+        wlmaker_subprocess_monitor_run(
+            server_ptr->monitor_ptr,
+            bs_subprocess_create_cmdline(arg_ptr));
         break;
 
     case WLMAKER_ACTION_WORKSPACE_TO_PREVIOUS:
@@ -229,6 +263,24 @@ void wlmaker_action_execute(wlmaker_server_t *server_ptr,
         wlmtk_root_switch_to_next_workspace(server_ptr->root_ptr);
         break;
 
+    case WLMAKER_ACTION_WORKSPACE_ADD:
+        workspace_ptr = wlmtk_workspace_create(
+            server_ptr->wlr_output_layout_ptr,
+            "New",
+            &server_ptr->style.tile);
+        if (NULL != workspace_ptr) {
+            BS_ASSERT_NOTNULL(wlmaker_background_create(
+                                  workspace_ptr,
+                                  server_ptr->wlr_output_layout_ptr,
+                                  server_ptr->style.background_color));
+            wlmtk_root_add_workspace(server_ptr->root_ptr, workspace_ptr);
+        }
+        break;
+
+    case WLMAKER_ACTION_WORKSPACE_DESTROY_LAST:
+        wlmtk_root_destroy_last_workspace(server_ptr->root_ptr);
+        break;
+
     case WLMAKER_ACTION_TASK_TO_PREVIOUS:
         wlmtk_workspace_activate_previous_window(
             wlmtk_root_get_current_workspace(server_ptr->root_ptr));
@@ -359,7 +411,8 @@ void wlmaker_action_execute(wlmaker_server_t *server_ptr,
             wlmtk_workspace_map_window(
                 wlmtk_root_get_current_workspace(server_ptr->root_ptr),
                 wlmaker_root_menu_window(server_ptr->root_menu_ptr));
-            wlmtk_window_set_position(
+            wlmtk_workspace_set_window_position(
+                wlmtk_root_get_current_workspace(server_ptr->root_ptr),
                 wlmaker_root_menu_window(server_ptr->root_menu_ptr),
                 server_ptr->cursor_ptr->wlr_cursor_ptr->x,
                 server_ptr->cursor_ptr->wlr_cursor_ptr->y);
@@ -375,6 +428,17 @@ void wlmaker_action_execute(wlmaker_server_t *server_ptr,
         }
         break;
 
+    case WLMAKER_ACTION_OUTPUT_MAGNIFY:
+        wlmbe_backend_magnify(server_ptr->backend_ptr);
+        break;
+    case WLMAKER_ACTION_OUTPUT_REDUCE:
+        wlmbe_backend_reduce(server_ptr->backend_ptr);
+        break;
+
+    case WLMAKER_ACTION_OUTPUT_SAVE_STATE:
+        wlmbe_backend_save_ephemeral_output_configs(server_ptr->backend_ptr);
+        break;
+
     case WLMAKER_ACTION_SWITCH_TO_VT1:
     case WLMAKER_ACTION_SWITCH_TO_VT2:
     case WLMAKER_ACTION_SWITCH_TO_VT3:
@@ -389,8 +453,8 @@ void wlmaker_action_execute(wlmaker_server_t *server_ptr,
     case WLMAKER_ACTION_SWITCH_TO_VT12:
         // Enums are required to be defined consecutively, so we can compute
         // the VT number from the action code.
-        _wlmaker_action_switch_to_vt(
-            server_ptr,
+        wlmbe_backend_switch_to_vt(
+            server_ptr->backend_ptr,
             action - WLMAKER_ACTION_SWITCH_TO_VT1 + 1);
         break;
 
@@ -415,11 +479,11 @@ void wlmaker_action_execute(wlmaker_server_t *server_ptr,
  */
 bool _wlmaker_keybindings_bind_item(
     const char *key_ptr,
-    wlmcfg_object_t *object_ptr,
+    bspl_object_t *object_ptr,
     void *userdata_ptr)
 {
     wlmaker_action_handle_t *handle_ptr = userdata_ptr;
-    wlmcfg_string_t *string_ptr = wlmcfg_string_from_object(object_ptr);
+    bspl_string_t *string_ptr = bspl_string_from_object(object_ptr);
     if (NULL == string_ptr) {
         bs_log(BS_WARNING, "Action must be a string for key binding \"%s\"",
                key_ptr);
@@ -431,15 +495,16 @@ bool _wlmaker_keybindings_bind_item(
     if (!_wlmaker_keybindings_parse(key_ptr, &modifiers, &keysym)) {
         bs_log(BS_WARNING,
                "Failed to parse binding '%s' for keybinding action '%s'",
-               key_ptr, wlmcfg_string_value(string_ptr));
+               key_ptr, bspl_string_value(string_ptr));
         return false;
     }
+    if (handle_ptr->add_logo) modifiers |= WLR_MODIFIER_LOGO;
 
     int action;
-    if (!wlmcfg_enum_name_to_value(
-            wlmaker_action_desc, wlmcfg_string_value(string_ptr), &action)) {
+    if (!bspl_enum_name_to_value(
+            wlmaker_action_desc, bspl_string_value(string_ptr), &action)) {
         bs_log(BS_WARNING, "Not a valid keybinding action: '%s'",
-               wlmcfg_string_value(string_ptr));
+               bspl_string_value(string_ptr));
         return false;
     }
 
@@ -500,7 +565,7 @@ bool _wlmaker_keybindings_parse(
         start_ptr = end_ptr;
 
         int new_modifier;
-        if (wlmcfg_enum_name_to_value(
+        if (bspl_enum_name_to_value(
                 _wlmaker_keybindings_modifiers, token_ptr, &new_modifier)) {
             *modifiers_ptr |= new_modifier;
         } else if (*keysym_ptr == XKB_KEY_NoSymbol) {
@@ -538,43 +603,21 @@ bool _wlmaker_action_bound_callback(const wlmaker_key_combo_t *key_combo_ptr)
     return true;
 }
 
-/* ------------------------------------------------------------------------- */
-/**
- * Switches to the given virtual terminal, if a wlroots session is available.
- *
- * Logs if wlr_session_change_vt() fails, but ignores the errors.
- *
- * @param server_ptr
- * @param vt_num
- */
-void _wlmaker_action_switch_to_vt(
-    wlmaker_server_t *server_ptr,
-    unsigned vt_num)
-{
-    // Guard clause: @ref wlmaker_server_t::session_ptr will be populated only
-    // if wlroots created a session, eg. when running from the terminal.
-    if (NULL == server_ptr->wlr_session_ptr) {
-        bs_log(BS_DEBUG, "_wlmaker_action_switch_to_vt: No session, ignored.");
-        return;
-    }
-
-    if (!wlr_session_change_vt(server_ptr->wlr_session_ptr, vt_num)) {
-        bs_log(BS_WARNING, "Failed wlr_session_change_vt(, %u)", vt_num);
-    }
-}
-
 /* == Unit tests =========================================================== */
 
 static void test_keybindings_parse(bs_test_t *test_ptr);
 static void test_default_keybindings(bs_test_t *test_ptr);
 
 /** Test cases for key bindings. */
-const bs_test_case_t          wlmaker_action_test_cases[] = {
-    { 1, "parse", test_keybindings_parse },
-    { 1, "default_keybindings", test_default_keybindings },
-    { 0, NULL, NULL }
+static const bs_test_case_t   wlmaker_action_test_cases[] = {
+    { true, "parse", test_keybindings_parse },
+    { true, "default_keybindings", test_default_keybindings },
+    BS_TEST_CASE_SENTINEL()
 };
 
+const bs_test_set_t wlmaker_action_test_set = BS_TEST_SET(
+    true, "action", wlmaker_action_test_cases);
+
 /* ------------------------------------------------------------------------- */
 /** Tests @ref _wlmaker_keybindings_parse. */
 void test_keybindings_parse(bs_test_t *test_ptr)
@@ -618,19 +661,19 @@ void test_keybindings_parse(bs_test_t *test_ptr)
 void test_default_keybindings(bs_test_t *test_ptr)
 {
     wlmaker_server_t server = {};
-    wlmcfg_object_t *obj_ptr = wlmcfg_create_object_from_plist_data(
+    bspl_object_t *obj_ptr = bspl_create_object_from_plist_data(
         embedded_binary_default_configuration_data,
         embedded_binary_default_configuration_size);
-    BS_TEST_VERIFY_NEQ(test_ptr, NULL, wlmcfg_dict_from_object(obj_ptr));
+    BS_TEST_VERIFY_NEQ(test_ptr, NULL, bspl_dict_from_object(obj_ptr));
 
-    wlmcfg_dict_t *dict_ptr = wlmcfg_dict_get_dict(
-        wlmcfg_dict_from_object(obj_ptr), wlmaker_action_config_dict_key);
+    bspl_dict_t *dict_ptr = bspl_dict_get_dict(
+        bspl_dict_from_object(obj_ptr), wlmaker_action_config_dict_key);
     BS_TEST_VERIFY_NEQ(test_ptr, NULL, dict_ptr);
 
     wlmaker_action_handle_t *handle_ptr = wlmaker_action_bind_keys(
-        &server, dict_ptr);
+        &server, dict_ptr, false);
     BS_TEST_VERIFY_NEQ(test_ptr, NULL, handle_ptr);
-    wlmcfg_object_unref(obj_ptr);
+    bspl_object_unref(obj_ptr);
     wlmaker_action_unbind_keys(handle_ptr);
 }
 
