@@ -25,11 +25,14 @@
 #include <stdbool.h>
 #include <stdint.h>
 #include <stdlib.h>
+#include <sys/stat.h>
 #include <toolkit/toolkit.h>
+#include <unistd.h>
 #include <wayland-server-core.h>
 #include <wayland-util.h>
 #define WLR_USE_UNSTABLE
 #include <wlr/backend.h>
+#include <wlr/backend/wayland.h>
 #include <wlr/backend/session.h>
 #include <wlr/render/allocator.h>
 #include <wlr/render/wlr_renderer.h>
@@ -39,7 +42,6 @@
 #include <wlr/types/wlr_scene.h>
 #include <wlr/types/wlr_screencopy_v1.h>
 #include <wlr/types/wlr_subcompositor.h>
-#include <wlr/version.h>
 #undef WLR_USE_UNSTABLE
 
 #include "output.h"
@@ -90,12 +92,17 @@ struct _wlmbe_backend_t {
     bs_dllist_t               output_configs;
     /**
      * Another list of @ref wlmbe_output_config_t items. This is
-     * initialized from wlmaker's state file.
+     * initialized from wlmaker's state file upon startup, ie. in
+     * @ref wlmbe_backend_create.
      *
      * If a discovered output equals to one of the nodes here, it's attributes
      * will be taken from here. Otherwise, a new entry is created in this list.
      * The intent is to memorize state of connected configs, so that
      * re-connected outputs are using the same attributes they left with.
+     *
+     * These configurations can be saved by calling
+     * @ref wlmbe_backend_save_ephemeral_output_configs, otherwise any updates
+     * will be lost on restart.
      */
     bs_dllist_t               ephemeral_output_configs;
 
@@ -107,6 +114,9 @@ struct _wlmbe_backend_t {
     struct wlr_scene          *wlr_scene_ptr;
     /** Points to struct wlr_output_layout. */
     struct wlr_output_layout  *wlr_output_layout_ptr;
+
+    /** Name of the file for storing the output's state. */
+    char                      *state_fname_ptr;
 };
 
 static void _wlmbe_backend_handle_new_output(
@@ -117,6 +127,9 @@ static bool _wlmbe_backend_decode_item(
     bspl_object_t *obj_ptr,
     size_t i,
     void *dest_ptr);
+static bspl_object_t *_wlmbe_backend_encode_all(
+    const union bspl_desc_value *desc_value_ptr,
+    const void *value_ptr);
 static void _wlmbe_backend_decode_fini(void *dest_ptr);
 static void _wlmbe_backend_config_dlnode_destroy(
     bs_dllist_node_t *dlnode_ptr,
@@ -130,19 +143,19 @@ static const bspl_desc_t _wlmbe_output_configs_desc[] = {
                     output_configs,
                     output_configs,
                     _wlmbe_backend_decode_item,
-                    NULL,
-                    NULL,
+                    _wlmbe_backend_encode_all,
+                    NULL,  // init.
                     _wlmbe_backend_decode_fini),
     BSPL_DESC_SENTINEL(),
 };
 
-/** Descriptor for the output state, stored as plist. */
+/** Descriptor for ephemeral output onfigurations, stored as plist. */
 static const bspl_desc_t _wlmbe_outputs_state_desc[] = {
     BSPL_DESC_ARRAY("Outputs", true, wlmbe_backend_t,
                     ephemeral_output_configs,
                     ephemeral_output_configs,
                     _wlmbe_backend_decode_item,
-                    NULL,
+                    _wlmbe_backend_encode_all,
                     NULL,
                     _wlmbe_backend_decode_fini),
     BSPL_DESC_SENTINEL(),
@@ -151,6 +164,11 @@ static const bspl_desc_t _wlmbe_outputs_state_desc[] = {
 /** Magnification factor: 3rd square root of 2.0. ~10, doubles exactly. */
 static const double _wlmbke_backend_magnification = 1.0905077326652577;
 
+/** The window title to set, when on a Wayland backend. */
+static const char *_wlmbe_wl_title = "wlmaker";
+/** The application ID to set, when on a Wayland backend. */
+static const char *_wlmbe_wl_app_id = "wlmaker";
+
 /* == Exported methods ===================================================== */
 
 /* ------------------------------------------------------------------------- */
@@ -160,7 +178,8 @@ wlmbe_backend_t *wlmbe_backend_create(
     struct wlr_output_layout *wlr_output_layout_ptr,
     int width,
     int height,
-    bspl_dict_t *config_dict_ptr)
+    bspl_dict_t *config_dict_ptr,
+    const char *state_fname_ptr)
 {
     wlmbe_backend_t *backend_ptr = logged_calloc(1, sizeof(wlmbe_backend_t));
     if (NULL == backend_ptr) return NULL;
@@ -177,13 +196,34 @@ wlmbe_backend_t *wlmbe_backend_create(
         return NULL;
     }
 
+    // Loads state (if available).
+    backend_ptr->state_fname_ptr = logged_strdup(state_fname_ptr);
+    if (NULL == backend_ptr->state_fname_ptr) {
+        wlmbe_backend_destroy(backend_ptr);
+        return NULL;
+    }
+    if (bs_file_realpath_is(backend_ptr->state_fname_ptr, S_IFREG)) {
+        bspl_object_t *o = bspl_create_object_from_plist_file(
+            backend_ptr->state_fname_ptr);
+        if (NULL == o) {
+            wlmbe_backend_destroy(backend_ptr);
+            return NULL;
+        }
+        bool rv = bspl_decode_dict(
+            bspl_dict_from_object(o),
+            _wlmbe_outputs_state_desc,
+            backend_ptr);
+        bspl_object_unref(o);
+        if (!rv) {
+            wlmbe_backend_destroy(backend_ptr);
+            return NULL;
+        }
+        bs_log(BS_INFO, "Loaded output state from \"%s\"", state_fname_ptr);
+    }
+
     // Auto-create the wlroots backend. Can be X11 or direct.
     backend_ptr->wlr_backend_ptr = wlr_backend_autocreate(
-#if WLR_VERSION_NUM >= (18 << 8)
         wl_display_get_event_loop(wl_display_ptr),
-#else // WLR_VERSION_NUM >= (18 << 8)
-        wl_display_ptr,
-#endif // WLR_VERSION_NUM >= (18 << 8)
         &backend_ptr->wlr_session_ptr);
     if (NULL == backend_ptr->wlr_backend_ptr) {
         bs_log(BS_ERROR, "Failed wlr_backend_autocreate()");
@@ -291,6 +331,11 @@ void wlmbe_backend_destroy(wlmbe_backend_t *backend_ptr)
 
     // @ref wlmbe_backend_t::wlr_backend_ptr is destroyed from wl_display.
 
+    if (NULL != backend_ptr->state_fname_ptr) {
+        free(backend_ptr->state_fname_ptr);
+        backend_ptr->state_fname_ptr = NULL;
+    }
+
     free(backend_ptr);
 }
 
@@ -356,6 +401,31 @@ void wlmbe_backend_reduce(wlmbe_backend_t *backend_ptr)
         1.0 / _wlmbke_backend_magnification);
 }
 
+/* ------------------------------------------------------------------------- */
+bool wlmbe_backend_save_ephemeral_output_configs(wlmbe_backend_t *backend_ptr)
+{
+    bspl_dict_t *dict_ptr = bspl_encode_dict(
+        _wlmbe_outputs_state_desc,
+        backend_ptr);
+    if (NULL != dict_ptr) {
+        bs_dynbuf_t dynbuf = {};
+        if (bs_dynbuf_init(&dynbuf, getpagesize(), SIZE_MAX)) {
+            if (bspl_object_write(
+                    bspl_object_from_dict(dict_ptr),
+                    &dynbuf) &&
+                bs_dynbuf_write_file(
+                    &dynbuf,
+                    backend_ptr->state_fname_ptr,
+                    S_IRUSR | S_IWUSR)) {
+                return true;
+            }
+            bs_dynbuf_fini(&dynbuf);
+        }
+        bspl_dict_unref(dict_ptr);
+    }
+    return false;
+}
+
 /* == Local (static) methods =============================================== */
 
 /* ------------------------------------------------------------------------- */
@@ -371,7 +441,7 @@ bool _wlmbe_backend_add_output(
     wlmbe_backend_t *backend_ptr,
     wlmbe_output_t *output_ptr)
 {
-    wlmbe_output_config_attributes_t *attr_ptr =
+    const wlmbe_output_config_attributes_t *attr_ptr =
         wlmbe_output_attributes(output_ptr);
     BS_ASSERT(NULL != attr_ptr);
 
@@ -393,8 +463,6 @@ bool _wlmbe_backend_add_output(
                wlrop->name);
         return false;
     }
-    attr_ptr->position.x = wlr_output_layout_output_ptr->x;
-    attr_ptr->position.y = wlr_output_layout_output_ptr->y;
 
     struct wlr_scene_output *wlr_scene_output_ptr = wlr_scene_output_create(
         backend_ptr->wlr_scene_ptr, wlrop);
@@ -408,11 +476,12 @@ bool _wlmbe_backend_add_output(
         wlmbe_dlnode_from_output(output_ptr));
 
     bs_log(BS_INFO,
-           "Created: Output <%s> %s to %dx%d@%.2f position (%d,%d) %s",
+           "Created: Output <%s> %s to %dx%d@%.2f x%.2f position (%d,%d) %s",
            wlmbe_output_description(output_ptr),
            wlrop->enabled ? "enabled" : "disabled",
            wlrop->width, wlrop->height,
            1e-3 * wlrop->refresh,
+           wlrop->scale,
            wlr_output_layout_output_ptr->x,
            wlr_output_layout_output_ptr->y,
            attr_ptr->has_position ? "explicit" : "auto");
@@ -429,6 +498,12 @@ void _wlmbe_backend_handle_new_output(
         listener_ptr, wlmbe_backend_t, new_output_listener);
     struct wlr_output *wlr_output_ptr = data_ptr;
 
+    // New output: If it's using a Wayland backend, set title & App ID.
+    if (wlr_output_is_wl(wlr_output_ptr)) {
+        wlr_wl_output_set_app_id(wlr_output_ptr, _wlmbe_wl_app_id);
+        wlr_wl_output_set_title(wlr_output_ptr, _wlmbe_wl_title);
+    }
+
     // See if there is an exact match among the ephemeral output configs. If
     // yes, pick that configuration. Otherwise, create a new one.
     wlmbe_output_config_t *config_ptr = wlmbe_output_config_from_dlnode(
@@ -442,24 +517,26 @@ void _wlmbe_backend_handle_new_output(
         wlr_output_destroy(wlr_output_ptr);
         return;
     }
+
     if (NULL == config_ptr) {
         config_ptr = wlmbe_output_config_create_from_wlr(wlr_output_ptr);
         bs_dllist_push_front(
             &backend_ptr->ephemeral_output_configs,
             wlmbe_dlnode_from_output_config(config_ptr));
-    }
 
-    // See if we have a corresponding entry among configured outputs. If yes,
-    // apply the attributes to our new config.
-    wlmbe_output_config_t *outputs_config_ptr =
-        wlmbe_output_config_from_dlnode(
-            bs_dllist_find(
-                &backend_ptr->output_configs,
-                wlmbe_output_config_fnmatches,
-                wlr_output_ptr));
-    if (NULL != outputs_config_ptr) {
-        *wlmbe_output_config_attributes(config_ptr) =
-            *wlmbe_output_config_attributes(outputs_config_ptr);
+        // See if we have a corresponding entry among configured outputs. If
+        // yes, apply the attributes to our new config.
+        wlmbe_output_config_t *outputs_config_ptr =
+            wlmbe_output_config_from_dlnode(
+                bs_dllist_find(
+                    &backend_ptr->output_configs,
+                    wlmbe_output_config_fnmatches,
+                    wlr_output_ptr));
+        if (NULL != outputs_config_ptr) {
+            wlmbe_output_config_apply_attributes(
+                config_ptr,
+            wlmbe_output_config_attributes(outputs_config_ptr));
+        }
     }
 
     wlmbe_output_t *output_ptr = wlmbe_output_create(
@@ -503,6 +580,45 @@ bool _wlmbe_backend_decode_item(
     return false;
 }
 
+/* ------------------------------------------------------------------------- */
+/** Encodes the dllist item and stores it into the array at `ud_ptr`. */
+bool _wlmbe_backend_encode_item(
+    bs_dllist_node_t *dlnode_ptr,
+    void *ud_ptr)
+{
+    bspl_array_t *array_ptr = ud_ptr;
+
+    wlmbe_output_config_t *c = wlmbe_output_config_from_dlnode(dlnode_ptr);
+
+    bspl_dict_t *dict_ptr = wlmbe_output_config_create_into_plist(c);
+    if (NULL == dict_ptr) return false;
+
+    bool rv = bspl_array_push_back(array_ptr, bspl_object_from_dict(dict_ptr));
+    bspl_dict_unref(dict_ptr);
+    return rv;
+}
+
+/* ------------------------------------------------------------------------- */
+/** Encodes all array (items) into a plist array. */
+bspl_object_t *_wlmbe_backend_encode_all(
+    __UNUSED__ const union bspl_desc_value *desc_value_ptr,
+    const void *value_ptr)
+{
+    const bs_dllist_t *output_configs_ptr = value_ptr;
+
+    bspl_array_t *array_ptr = bspl_array_create();
+    if (NULL != array_ptr &&
+        bs_dllist_all(
+            output_configs_ptr,
+            _wlmbe_backend_encode_item,
+            array_ptr)) {
+        return bspl_object_from_array(array_ptr);
+    }
+
+    bspl_array_unref(array_ptr);
+    return NULL;
+}
+
 /* ------------------------------------------------------------------------- */
 /** Frees all resources of the @ref wlmbe_output_config_t in the list. */
 void _wlmbe_backend_decode_fini(void *dest_ptr)
@@ -531,9 +647,11 @@ void _wlmbe_backend_config_dlnode_destroy(
 /* == Unit tests =========================================================== */
 
 static void _wlmbe_backend_test_find(bs_test_t *test_ptr);
+static void _wlmbe_backend_test_encode_output_configs(bs_test_t *test_ptr);
 
 const bs_test_case_t          wlmbe_backend_test_cases[] = {
     { 1, "find", _wlmbe_backend_test_find },
+    { 1, "encode_output_configs", _wlmbe_backend_test_encode_output_configs },
     { 0, NULL, NULL }
 };
 
@@ -604,4 +722,77 @@ void _wlmbe_backend_test_find(bs_test_t *test_ptr)
     bspl_dict_unref(config_dict_ptr);
 }
 
+/* ------------------------------------------------------------------------- */
+/** Tests encoding of the ephemeral output configs */
+void _wlmbe_backend_test_encode_output_configs(bs_test_t *test_ptr)
+{
+    wlmbe_backend_t backend = {};
+
+    struct wlr_output w = {
+        .name = "Name0",
+        .width = 1024,
+        .height = 768,
+        .scale = 2.0,
+        .refresh = 60000,
+    };
+
+    wlmbe_output_config_t *c1 = wlmbe_output_config_create_from_wlr(&w);
+    BS_TEST_VERIFY_NEQ_OR_RETURN(test_ptr, NULL, c1);
+    bs_dllist_push_back(
+        &backend.ephemeral_output_configs,
+        wlmbe_dlnode_from_output_config(c1));
+
+    w = (struct wlr_output){
+        .enabled = true,
+        .name = "Name1",
+        .width = 640,
+        .height = 480,
+        .scale = 1.0,
+        .refresh = 80000
+    };
+    wlmbe_output_config_t *c2 = wlmbe_output_config_create_from_wlr(&w);
+    BS_TEST_VERIFY_NEQ_OR_RETURN(test_ptr, NULL, c2);
+    bs_dllist_push_back(
+        &backend.ephemeral_output_configs,
+        wlmbe_dlnode_from_output_config(c2));
+
+
+    bspl_dict_t *dict_ptr = bspl_encode_dict(
+        _wlmbe_outputs_state_desc,
+        &backend);
+    BS_TEST_VERIFY_NEQ_OR_RETURN(test_ptr, NULL, dict_ptr);
+
+    bs_dynbuf_t dynbuf = {};
+    BS_TEST_VERIFY_TRUE(test_ptr, bs_dynbuf_init(&dynbuf, 1024, SIZE_MAX));
+    BS_TEST_VERIFY_TRUE(
+        test_ptr,
+        bspl_object_write(bspl_object_from_dict(dict_ptr), &dynbuf));
+
+    static const char *expected =
+        "{\n"
+        "  Outputs = (\n"
+        "    {\n"
+        "      Enabled = False;\n"
+        "      Mode = \"1024x768@60.0\";\n"
+        "      Name = Name0;\n"
+        "      Scale = \"2.000000e+00\";\n"
+        "      Transformation = Normal;\n"
+        "    },\n"
+        "    {\n"
+        "      Enabled = True;\n"
+        "      Mode = \"640x480@80.0\";\n"
+        "      Name = Name1;\n"
+        "      Scale = \"1.000000e+00\";\n"
+        "      Transformation = Normal;\n"
+        "    }\n"
+        "  );\n"
+        "}\n";
+    BS_TEST_VERIFY_MEMEQ(test_ptr, expected, dynbuf.data_ptr, dynbuf.length);
+
+    bs_dynbuf_fini(&dynbuf);
+    bspl_dict_unref(dict_ptr);
+    wlmbe_output_config_destroy(c2);
+    wlmbe_output_config_destroy(c1);
+}
+
 /* == End of backend.c ===================================================== */
