@@ -20,8 +20,25 @@
 
 #include "clip.h"
 
-#include <limits.h>
+#include <cairo.h>
+#include <libbase/libbase.h>
+#include <libbase/plist.h>
+#include <linux/input-event-codes.h>
+#include <stdbool.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <sys/stat.h>
+#include <wayland-server-core.h>
+#include <wayland-util.h>
+#define WLR_USE_UNSTABLE
+#include <wlr/types/wlr_output.h>
+#include <wlr/types/wlr_output_layout.h>
+#include <wlr/util/edges.h>
+#undef WLR_USE_UNSTABLE
 
+#include "backend/backend.h"
+#include "backend/output_config.h"
+#include "files.h"
 #include "toolkit/toolkit.h"
 
 /* == Declarations ========================================================= */
@@ -51,6 +68,9 @@ struct _wlmaker_clip_t {
     /** Clip image. */
     wlmtk_image_t             *image_ptr;
 
+    /** Description of the desired output, if any. */
+    wlmbe_output_description_t output_description;
+
     /** Whether the pointer is currently inside the 'prev' button. */
     bool                      pointer_inside_prev_button;
     /** Whether the pointer is currently inside the 'next' button. */
@@ -62,6 +82,12 @@ struct _wlmaker_clip_t {
 
     /** Listener for @ref wlmtk_root_events_t::workspace_changed. */
     struct wl_listener        workspace_changed_listener;
+    /** Listener for wlr_output_layout::events.change. */
+    struct wl_listener        output_layout_change_listener;
+    /** Listener for @ref wlmtk_element_events_t::pointer_motion. */
+    struct wl_listener        pointer_motion_listener;
+    /** Listener for @ref wlmtk_element_events_t::pointer_leave. */
+    struct wl_listener        pointer_leave_listener;
 
     /** The clip's style. */
     wlmaker_config_clip_style_t style;
@@ -73,12 +99,6 @@ static bool _wlmaker_clip_pointer_axis(
 static bool _wlmaker_clip_pointer_button(
     wlmtk_element_t *element_ptr,
     const wlmtk_button_event_t *button_event_ptr);
-static bool _wlmaker_clip_pointer_motion(
-    wlmtk_element_t *element_ptr,
-    double x, double y,
-    uint32_t time_msec);
-static void _wlmaker_clip_pointer_leave(
-    wlmtk_element_t *element_ptr);
 
 static void _wlmaker_clip_update_buttons(wlmaker_clip_t *clip_ptr);
 static void _wlmaker_clip_update_overlay(wlmaker_clip_t *clip_ptr);
@@ -90,6 +110,15 @@ static struct wlr_buffer *_wlmaker_clip_create_tile(
 static void _wlmaker_clip_handle_workspace_changed(
     struct wl_listener *listener_ptr,
     void *data_ptr);
+static void _wlmaker_clip_handle_output_layout_change(
+    struct wl_listener *listener_ptr,
+    void *data_ptr);
+static void _wlmaker_clip_handle_pointer_motion(
+    struct wl_listener *listener_ptr,
+    void *data_ptr);
+static void _wlmaker_clip_handle_pointer_leave(
+    struct wl_listener *listener_ptr,
+    void *data_ptr);
 
 /* == Data ================================================================= */
 
@@ -97,8 +126,6 @@ static void _wlmaker_clip_handle_workspace_changed(
 static const wlmtk_element_vmt_t _wlmaker_clip_element_vmt = {
     .pointer_axis = _wlmaker_clip_pointer_axis,
     .pointer_button = _wlmaker_clip_pointer_button,
-    .pointer_motion = _wlmaker_clip_pointer_motion,
-    .pointer_leave = _wlmaker_clip_pointer_leave,
 };
 
 /** TODO: Replace this. */
@@ -108,34 +135,23 @@ typedef struct {
 } parse_args;
 
 /** Enum descriptor for `enum wlr_edges`. */
-static const wlmcfg_enum_desc_t _wlmaker_clip_edges[] = {
-    WLMCFG_ENUM("TOP", WLR_EDGE_TOP),
-    WLMCFG_ENUM("BOTTOM", WLR_EDGE_BOTTOM),
-    WLMCFG_ENUM("LEFT", WLR_EDGE_LEFT),
-    WLMCFG_ENUM("RIGHT", WLR_EDGE_RIGHT),
-    WLMCFG_ENUM_SENTINEL(),
+static const bspl_enum_desc_t _wlmaker_clip_edges[] = {
+    BSPL_ENUM("TOP", WLR_EDGE_TOP),
+    BSPL_ENUM("BOTTOM", WLR_EDGE_BOTTOM),
+    BSPL_ENUM("LEFT", WLR_EDGE_LEFT),
+    BSPL_ENUM("RIGHT", WLR_EDGE_RIGHT),
+    BSPL_ENUM_SENTINEL(),
 };
 
 /** Descriptor for the clip's plist. */
-const wlmcfg_desc_t _wlmaker_clip_desc[] = {
-    WLMCFG_DESC_ENUM("Edge", true, parse_args, positioning.edge,
-                     WLR_EDGE_NONE, _wlmaker_clip_edges),
-    WLMCFG_DESC_ENUM("Anchor", true, parse_args, positioning.anchor,
-                     WLR_EDGE_NONE, _wlmaker_clip_edges),
-    WLMCFG_DESC_SENTINEL(),
-};
-
-/** Lookup paths for icons -- FIXME: de-duplicate this! */
-static const char *lookup_paths[] = {
-    "/usr/share/icons/wlmaker",
-    "/usr/local/share/icons/wlmaker",
-#if defined(WLMAKER_SOURCE_DIR)
-    WLMAKER_SOURCE_DIR "/icons",
-#endif  // WLMAKER_SOURCE_DIR
-#if defined(WLMAKER_ICON_DATA_DIR)
-    WLMAKER_ICON_DATA_DIR,
-#endif  // WLMAKER_ICON_DATA_DIR
-    NULL
+const bspl_desc_t _wlmaker_clip_desc[] = {
+    BSPL_DESC_ENUM("Edge", true, parse_args,
+                   positioning.edge, positioning.edge,
+                   WLR_EDGE_NONE, _wlmaker_clip_edges),
+    BSPL_DESC_ENUM("Anchor", true, parse_args,
+                   positioning.anchor, positioning.anchor,
+                   WLR_EDGE_NONE, _wlmaker_clip_edges),
+    BSPL_DESC_SENTINEL(),
 };
 
 /* == Exported methods ===================================================== */
@@ -143,7 +159,7 @@ static const char *lookup_paths[] = {
 /* ------------------------------------------------------------------------- */
 wlmaker_clip_t *wlmaker_clip_create(
     wlmaker_server_t *server_ptr,
-    wlmcfg_dict_t *state_dict_ptr,
+    bspl_dict_t *state_dict_ptr,
     const wlmaker_config_style_t *style_ptr)
 {
     wlmaker_clip_t *clip_ptr = logged_calloc(1, sizeof(wlmaker_clip_t));
@@ -165,66 +181,98 @@ wlmaker_clip_t *wlmaker_clip_create(
     }
 
     parse_args args = {};
-    wlmcfg_dict_t *dict_ptr = wlmcfg_dict_get_dict(state_dict_ptr, "Clip");
+    bspl_dict_t *dict_ptr = bspl_dict_get_dict(state_dict_ptr, "Clip");
     if (NULL == dict_ptr) {
         bs_log(BS_ERROR, "No 'Clip' dict found in state.");
         wlmaker_clip_destroy(clip_ptr);
         return NULL;
     }
-    wlmcfg_decode_dict(dict_ptr, _wlmaker_clip_desc, &args);
+    bspl_decode_dict(dict_ptr, _wlmaker_clip_desc, &args);
+    bspl_dict_t *output_dict_ptr = bspl_dict_get_dict(dict_ptr, "Output");
+    if (NULL != output_dict_ptr) {
+        if (!wlmbe_output_description_init_from_plist(
+                &clip_ptr->output_description, output_dict_ptr)) {
+            wlmaker_clip_destroy(clip_ptr);
+            return NULL;
+        }
+    }
 
     clip_ptr->wlmtk_dock_ptr = wlmtk_dock_create(
-        &args.positioning, &style_ptr->dock, server_ptr->env_ptr);
+        &args.positioning, &style_ptr->dock);
     wlmtk_element_set_visible(
         wlmtk_dock_element(clip_ptr->wlmtk_dock_ptr),
         true);
 
     if (!wlmtk_tile_init(
             &clip_ptr->super_tile,
-            &style_ptr->tile,
-            server_ptr->env_ptr)) {
+            &style_ptr->tile)) {
         wlmaker_clip_destroy(clip_ptr);
         return NULL;
     }
     clip_ptr->orig_super_element_vmt = wlmtk_element_extend(
         wlmtk_tile_element(&clip_ptr->super_tile),
         &_wlmaker_clip_element_vmt);
+    wlmtk_util_connect_listener_signal(
+        &wlmtk_tile_element(&clip_ptr->super_tile)->events.pointer_motion,
+        &clip_ptr->pointer_motion_listener,
+        _wlmaker_clip_handle_pointer_motion);
+    wlmtk_util_connect_listener_signal(
+        &wlmtk_tile_element(&clip_ptr->super_tile)->events.pointer_leave,
+        &clip_ptr->pointer_leave_listener,
+        _wlmaker_clip_handle_pointer_leave);
+
     wlmtk_element_set_visible(
         wlmtk_tile_element(&clip_ptr->super_tile), true);
     wlmtk_tile_set_background_buffer(
         &clip_ptr->super_tile, clip_ptr->tile_buffer_ptr);
     wlmtk_dock_add_tile(clip_ptr->wlmtk_dock_ptr, &clip_ptr->super_tile);
 
-    if (!wlmtk_buffer_init(
-            &clip_ptr->overlay_buffer, server_ptr->env_ptr)) {
+    if (!wlmtk_buffer_init(&clip_ptr->overlay_buffer)) {
         wlmaker_clip_destroy(clip_ptr);
         return NULL;
     }
     wlmtk_element_set_visible(
         wlmtk_buffer_element(&clip_ptr->overlay_buffer), true);
 
+    struct wlr_output *wlr_output_ptr = wlmbe_output_description_first_fnmatch(
+        &clip_ptr->output_description, server_ptr->wlr_output_layout_ptr);
+    if (NULL == wlr_output_ptr) {
+        wlr_output_ptr = wlmbe_primary_output(
+            server_ptr->wlr_output_layout_ptr);
+    }
+
     wlmtk_workspace_t *workspace_ptr =
         wlmtk_root_get_current_workspace(server_ptr->root_ptr);
     wlmtk_layer_t *layer_ptr = wlmtk_workspace_get_layer(
         workspace_ptr, WLMTK_WORKSPACE_LAYER_TOP);
-    wlmtk_layer_add_panel(
-        layer_ptr,
-        wlmtk_dock_panel(clip_ptr->wlmtk_dock_ptr));
-
-    char full_path[PATH_MAX];
-    char *path_ptr = bs_file_resolve_and_lookup_from_paths(
-        "clip-48x48.png", lookup_paths, 0, full_path);
-    if (NULL == path_ptr) {
-        bs_log(BS_ERROR | BS_ERRNO,
-               "Failed bs_file_resolve_and_lookup_from_paths(\"clip-48x48.png\" ...)");
+    if (!wlmtk_layer_add_panel(
+            layer_ptr,
+            wlmtk_dock_panel(clip_ptr->wlmtk_dock_ptr),
+            wlr_output_ptr)) {
         wlmaker_clip_destroy(clip_ptr);
         return NULL;
     }
+
+    // Resolves to a full path, and verifies the icon file exists.
+    char *path_ptr = wlmaker_files_xdg_data_find(
+        server_ptr->files_ptr, "icons/clip-48x48.png", S_IFREG);
+    const char *p1 = path_ptr;
+    if (NULL == path_ptr) {
+        bs_log(
+            BS_WARNING,
+            "Failed to locate ${XDG_DATA_DIRS}/wlmaker/icons/clip-48x48.png");
+#ifndef WLMAKER_SOURCE_DIR
+        wlmaker_clip_destroy(clip_ptr);
+        return NULL;
+#else
+        p1 = WLMAKER_SOURCE_DIR "/share/wlmaker/icons/clip-48x48.png";
+#endif
+    }
     clip_ptr->image_ptr = wlmtk_image_create_scaled(
-        path_ptr,
+        p1,
         clip_ptr->super_tile.style.content_size,
-        clip_ptr->super_tile.style.content_size,
-        server_ptr->env_ptr);
+        clip_ptr->super_tile.style.content_size);
+    free(path_ptr);
     if (NULL == clip_ptr->image_ptr) {
         wlmaker_clip_destroy(clip_ptr);
         return NULL;
@@ -245,6 +293,15 @@ wlmaker_clip_t *wlmaker_clip_create(
         &clip_ptr->workspace_changed_listener,
         _wlmaker_clip_handle_workspace_changed);
 
+    // TODO(kaeser@gubbe.ch): This is a very hacky way of updating the output
+    // before the layer's handler removes all associated panels. Should be
+    // a native method of wlmtk_dock_t or wlmtk_panel_t.
+    clip_ptr->output_layout_change_listener.notify =
+        _wlmaker_clip_handle_output_layout_change;
+    wl_list_insert(
+        server_ptr->wlr_output_layout_ptr->events.change.listener_list.next,
+        &clip_ptr->output_layout_change_listener.link);
+
     server_ptr->clip_dock_ptr = clip_ptr->wlmtk_dock_ptr;
     bs_log(BS_INFO, "Created clip %p", clip_ptr);
     return clip_ptr;
@@ -257,6 +314,7 @@ void wlmaker_clip_destroy(wlmaker_clip_t *clip_ptr)
         clip_ptr->server_ptr->clip_dock_ptr = NULL;
     }
 
+    wlmtk_util_disconnect_listener(&clip_ptr->output_layout_change_listener);
     wlmtk_util_disconnect_listener(&clip_ptr->workspace_changed_listener);
 
     if (wlmtk_tile_element(&clip_ptr->super_tile)->parent_container_ptr) {
@@ -266,6 +324,8 @@ void wlmaker_clip_destroy(wlmaker_clip_t *clip_ptr)
             clip_ptr->wlmtk_dock_ptr,
             &clip_ptr->super_tile);
     }
+    wlmtk_util_disconnect_listener(&clip_ptr->pointer_leave_listener);
+    wlmtk_util_disconnect_listener(&clip_ptr->pointer_motion_listener);
     wlmtk_tile_fini(&clip_ptr->super_tile);
     wlmtk_buffer_fini(&clip_ptr->overlay_buffer);
 
@@ -275,10 +335,12 @@ void wlmaker_clip_destroy(wlmaker_clip_t *clip_ptr)
     }
 
     if (NULL != clip_ptr->wlmtk_dock_ptr) {
-        wlmtk_layer_remove_panel(
-            wlmtk_panel_get_layer(wlmtk_dock_panel(clip_ptr->wlmtk_dock_ptr)),
-            wlmtk_dock_panel(clip_ptr->wlmtk_dock_ptr));
-
+        if (NULL != wlmtk_panel_get_layer(
+                wlmtk_dock_panel(clip_ptr->wlmtk_dock_ptr))) {
+            wlmtk_layer_remove_panel(
+                wlmtk_panel_get_layer(wlmtk_dock_panel(clip_ptr->wlmtk_dock_ptr)),
+                wlmtk_dock_panel(clip_ptr->wlmtk_dock_ptr));
+        }
         wlmtk_dock_destroy(clip_ptr->wlmtk_dock_ptr);
         clip_ptr->wlmtk_dock_ptr = NULL;
     }
@@ -296,6 +358,7 @@ void wlmaker_clip_destroy(wlmaker_clip_t *clip_ptr)
         clip_ptr->next_pressed_tile_buffer_ptr = NULL;
     }
 
+    wlmbe_output_description_fini(&clip_ptr->output_description);
     free(clip_ptr);
 }
 
@@ -356,10 +419,12 @@ bool _wlmaker_clip_pointer_button(
     switch (button_event_ptr->type) {
     case WLMTK_BUTTON_DOWN:
         // Pointer button tressed. Translate to button press if in area.
-        if (clip_ptr->pointer_inside_next_button) {
+        if (clip_ptr->pointer_inside_next_button ||
+            clip_ptr->pointer_inside_next_button) {
             clip_ptr->next_button_pressed = true;
             clip_ptr->prev_button_pressed = false;
-        } else if (clip_ptr->pointer_inside_prev_button) {
+        } else if (clip_ptr->pointer_inside_prev_button ||
+                   clip_ptr->pointer_inside_prev_button) {
             clip_ptr->next_button_pressed = false;
             clip_ptr->prev_button_pressed = true;
         }
@@ -368,12 +433,14 @@ bool _wlmaker_clip_pointer_button(
     case WLMTK_BUTTON_UP:
         // Button is released (closed the click). If we're within the area of
         // the pressed button: Trigger the action.
-        if (clip_ptr->pointer_inside_next_button &&
+        if ((clip_ptr->pointer_inside_next_button ||
+             clip_ptr->pointer_inside_next_button) &&
             clip_ptr->next_button_pressed) {
             clip_ptr->next_button_pressed = false;
             wlmtk_root_switch_to_next_workspace(
                 clip_ptr->server_ptr->root_ptr);
-        } else if (clip_ptr->pointer_inside_prev_button &&
+        } else if ((clip_ptr->pointer_inside_prev_button ||
+                    clip_ptr->pointer_inside_prev_button) &&
                    clip_ptr->prev_button_pressed) {
             clip_ptr->prev_button_pressed = false;
             wlmtk_root_switch_to_previous_workspace(
@@ -390,74 +457,17 @@ bool _wlmaker_clip_pointer_button(
     return true;
 }
 
-/* ------------------------------------------------------------------------- */
-/**
- * Implements @ref wlmtk_element_vmt_t::pointer_motion.
- *
- * Tracks whether the pointer is within any of the 'Next' or 'Previous' button
- * areas, and triggers an update to the tile's texture.
- *
- * @param element_ptr
- * @param x
- * @param y
- * @param time_msec
- *
- * @return See @ref wlmtk_element_vmt_t::pointer_motion.
- */
-bool _wlmaker_clip_pointer_motion(
-    wlmtk_element_t *element_ptr,
-    double x, double y,
-    uint32_t time_msec)
-{
-    wlmaker_clip_t *clip_ptr = BS_CONTAINER_OF(
-        element_ptr, wlmaker_clip_t,
-        super_tile.super_container.super_element);
-
-    clip_ptr->pointer_inside_prev_button = false;
-    clip_ptr->pointer_inside_next_button = false;
-
-    double tile_size = clip_ptr->super_tile.style.size;
-    double button_size = (22.0 / 64.0) * tile_size;
-    if (x >= tile_size - button_size && x < tile_size &&
-        y >= 0 && y < button_size) {
-        // Next button.
-        clip_ptr->pointer_inside_next_button = true;
-    } else if (x >= 0 && x < button_size &&
-               y >= tile_size - button_size && y < tile_size) {
-        // Prev button.
-        clip_ptr->pointer_inside_prev_button = true;
-    }
-
-    _wlmaker_clip_update_buttons(clip_ptr);
-    return clip_ptr->orig_super_element_vmt.pointer_motion(
-        element_ptr, x, y, time_msec);
-}
-
-/* ------------------------------------------------------------------------- */
-/** Implements @ref wlmtk_element_vmt_t::pointer_leave. Updates texture. */
-void _wlmaker_clip_pointer_leave(
-    wlmtk_element_t *element_ptr)
-{
-    wlmaker_clip_t *clip_ptr = BS_CONTAINER_OF(
-        element_ptr, wlmaker_clip_t,
-        super_tile.super_container.super_element);
-
-    clip_ptr->pointer_inside_prev_button = false;
-    clip_ptr->pointer_inside_next_button = false;
-    _wlmaker_clip_update_buttons(clip_ptr);
-    clip_ptr->orig_super_element_vmt.pointer_leave(element_ptr);
-}
-
-
 /* ------------------------------------------------------------------------- */
 /** Updates the button textures, based on current state what's pressed. */
 static void _wlmaker_clip_update_buttons(wlmaker_clip_t *clip_ptr)
 {
     struct wlr_buffer *wlr_buffer_ptr = clip_ptr->tile_buffer_ptr;
-    if (clip_ptr->pointer_inside_next_button &&
+    if ((clip_ptr->pointer_inside_next_button ||
+         clip_ptr->pointer_inside_next_button)&&
         clip_ptr->next_button_pressed) {
         wlr_buffer_ptr = clip_ptr->next_pressed_tile_buffer_ptr;
-    } else if (clip_ptr->pointer_inside_prev_button &&
+    } else if ((clip_ptr->pointer_inside_prev_button ||
+                clip_ptr->pointer_inside_prev_button) &&
                clip_ptr->prev_button_pressed) {
         wlr_buffer_ptr = clip_ptr->prev_pressed_tile_buffer_ptr;
     }
@@ -727,20 +737,100 @@ void _wlmaker_clip_handle_workspace_changed(
     if (NULL != current_layer_ptr) {
         wlmtk_layer_remove_panel(current_layer_ptr, panel_ptr);
     }
-    wlmtk_layer_add_panel(new_layer_ptr, panel_ptr);
+    BS_ASSERT(wlmtk_layer_add_panel(
+                  new_layer_ptr,
+                  panel_ptr,
+                  wlmbe_primary_output(
+                      clip_ptr->server_ptr->wlr_output_layout_ptr)));
 
     _wlmaker_clip_update_overlay(clip_ptr);
 }
 
+/* ------------------------------------------------------------------------- */
+/** Handles when output layout changes; Re-computes the output to attach. */
+void _wlmaker_clip_handle_output_layout_change(
+    struct wl_listener *listener_ptr,
+    __UNUSED__ void *data_ptr)
+{
+    wlmaker_clip_t *clip_ptr = BS_CONTAINER_OF(
+        listener_ptr, wlmaker_clip_t, output_layout_change_listener);
+
+    struct wlr_output *wlr_output_ptr = wlmbe_output_description_first_fnmatch(
+        &clip_ptr->output_description, clip_ptr->server_ptr->wlr_output_layout_ptr);
+    if (NULL == wlr_output_ptr) {
+        wlr_output_ptr = wlmbe_primary_output(
+            clip_ptr->server_ptr->wlr_output_layout_ptr);
+    }
+    wlmtk_layer_t *layer_ptr = wlmtk_panel_get_layer(
+        wlmtk_dock_panel(clip_ptr->wlmtk_dock_ptr));
+    wlmtk_layer_remove_panel(layer_ptr, wlmtk_dock_panel(clip_ptr->wlmtk_dock_ptr));
+    if (NULL != wlr_output_ptr) {
+        BS_ASSERT(wlmtk_layer_add_panel(
+                      layer_ptr,
+                      wlmtk_dock_panel(clip_ptr->wlmtk_dock_ptr),
+                      wlr_output_ptr));
+    }
+}
+
+/* ------------------------------------------------------------------------- */
+/** Handles @ref wlmtk_element_events_t::pointer_leave. Resets buttons. */
+void _wlmaker_clip_handle_pointer_leave(
+    struct wl_listener *listener_ptr,
+    __UNUSED__ void *data_ptr)
+{
+    wlmaker_clip_t *clip_ptr = BS_CONTAINER_OF(
+        listener_ptr, wlmaker_clip_t, pointer_leave_listener);
+
+    clip_ptr->pointer_inside_prev_button = false;
+    clip_ptr->pointer_inside_next_button = false;
+    _wlmaker_clip_update_buttons(clip_ptr);
+}
+
+/* ------------------------------------------------------------------------- */
+/** Handles @ref wlmtk_element_events_t::pointer_motion. */
+void _wlmaker_clip_handle_pointer_motion(
+    struct wl_listener *listener_ptr,
+    void *data_ptr)
+{
+    wlmaker_clip_t *clip_ptr = BS_CONTAINER_OF(
+        listener_ptr, wlmaker_clip_t, pointer_motion_listener);
+    wlmtk_pointer_motion_event_t *motion_event_ptr = data_ptr;
+
+    clip_ptr->pointer_inside_prev_button = false;
+    clip_ptr->pointer_inside_next_button = false;
+
+    double tile_size = clip_ptr->super_tile.style.size;
+    double button_size = (22.0 / 64.0) * tile_size;
+    if (motion_event_ptr->x >= tile_size - button_size &&
+        motion_event_ptr->x < tile_size &&
+        motion_event_ptr->y >= 0 &&
+        motion_event_ptr->y < button_size) {
+        // Next button.
+        clip_ptr->pointer_inside_next_button = true;
+    } else if (motion_event_ptr->x >= 0 &&
+               motion_event_ptr->x < button_size &&
+               motion_event_ptr->y >= tile_size - button_size &&
+               motion_event_ptr->y < tile_size) {
+        // Prev button.
+        clip_ptr->pointer_inside_prev_button = true;
+    }
+
+    _wlmaker_clip_update_buttons(clip_ptr);
+}
+
 /* == Unit tests =========================================================== */
 
 static void test_draw_tile(bs_test_t *test_ptr);
 
-const bs_test_case_t wlmaker_clip_test_cases[] = {
-    { 1, "draw_tile", test_draw_tile },
-    { 0, NULL, NULL }
+/** Test cases. */
+static const bs_test_case_t wlmaker_clip_test_cases[] = {
+    { true, "draw_tile", test_draw_tile },
+    BS_TEST_CASE_SENTINEL()
 };
 
+const bs_test_set_t wlmaker_clip_test_set = BS_TEST_SET(
+    true, "clip", wlmaker_clip_test_cases);
+
 /* ------------------------------------------------------------------------- */
 /** Tests that the clip tile is drawn correctly. */
 void test_draw_tile(bs_test_t *test_ptr)
