@@ -20,12 +20,20 @@
 
 #include "layer_panel.h"
 
-#include "wlr-layer-shell-unstable-v1-protocol.h"
-
+#include <inttypes.h>
+#include <libbase/libbase.h>
+#include <stdbool.h>
+#include <stdlib.h>
+#include <wayland-server-core.h>
+#include <wayland-server-protocol.h>
 #define WLR_USE_UNSTABLE
+#include <wlr/types/wlr_compositor.h>
 #include <wlr/types/wlr_layer_shell_v1.h>
+#include <wlr/util/edges.h>
 #undef WLR_USE_UNSTABLE
 
+#include "toolkit/toolkit.h"
+#include "wlr-layer-shell-unstable-v1-protocol.h"
 #include "xdg_popup.h"
 
 /* == Declarations ========================================================= */
@@ -56,12 +64,15 @@ struct _wlmaker_layer_panel_t {
     struct wl_listener        new_popup_listener;
 };
 
-static wlmaker_layer_panel_t *_wlmaker_layer_panel_create_injected(
+wlmaker_layer_panel_t *_wlmaker_layer_panel_create_from_surface(
     struct wlr_layer_surface_v1 *wlr_layer_surface_v1_ptr,
     wlmaker_server_t *server_ptr,
-    wlmtk_surface_create_t wlmtk_surface_create_fn);
-static void _wlmaker_layer_panel_destroy(
+    wlmtk_surface_t *wlmtk_surface_ptr);
+
+    static void _wlmaker_layer_panel_destroy(
     wlmaker_layer_panel_t *layer_panel_ptr);
+static void _wlmaker_layer_panel_element_destroy(
+    wlmtk_element_t *element_ptr);
 
 static bool _wlmaker_layer_panel_apply_keyboard(
     wlmaker_layer_panel_t *layer_panel_ptr,
@@ -96,9 +107,13 @@ static void _wlmaker_layer_panel_handle_new_popup(
 /* == Data ================================================================= */
 
 /** Virtual method table for the layer panel. */
-static const wlmtk_panel_vmt_t _wlmtk_layer_panel_vmt = {
+static const wlmtk_panel_vmt_t _wlmaker_layer_panel_vmt = {
     .request_size = _wlmaker_layer_panel_request_size,
 };
+/** Virtual method table for the layer panel's superclass element. */
+static const wlmtk_element_vmt_t _wlmaker_layer_panel_element_vmt = {
+    .destroy = _wlmaker_layer_panel_element_destroy,
+};
 
 /* == Exported methods ===================================================== */
 
@@ -107,52 +122,60 @@ wlmaker_layer_panel_t *wlmaker_layer_panel_create(
     struct wlr_layer_surface_v1 *wlr_layer_surface_v1_ptr,
     wlmaker_server_t *server_ptr)
 {
-    return _wlmaker_layer_panel_create_injected(
-        wlr_layer_surface_v1_ptr,
-        server_ptr,
-        wlmtk_surface_create);
+    wlmtk_surface_t *surface_ptr = wlmtk_surface_create(
+        wlr_layer_surface_v1_ptr->surface,
+        server_ptr->wlr_seat_ptr);
+    if (NULL == surface_ptr) return NULL;
+
+    wlmaker_layer_panel_t *panel_ptr =
+        _wlmaker_layer_panel_create_from_surface(
+            wlr_layer_surface_v1_ptr,
+            server_ptr,
+            surface_ptr);
+    if (NULL == panel_ptr) {
+        wlmtk_surface_destroy(surface_ptr);
+        return NULL;
+    }
+
+    return panel_ptr;
 }
 
 /* == Local (static) methods =============================================== */
 
 /* ------------------------------------------------------------------------- */
 /**
- * Constructor for the layer panel, with injectable methods.
+ * Constructor for the layer panel, with already-created surface.
  *
  * @param wlr_layer_surface_v1_ptr
  * @param server_ptr
- * @param wlmtk_surface_create_fn
+ * @param wlmtk_surface_ptr
  *
  * @return The handler for the layer surface or NULL on error.
  */
-wlmaker_layer_panel_t *_wlmaker_layer_panel_create_injected(
+wlmaker_layer_panel_t *_wlmaker_layer_panel_create_from_surface(
     struct wlr_layer_surface_v1 *wlr_layer_surface_v1_ptr,
     wlmaker_server_t *server_ptr,
-    wlmtk_surface_create_t wlmtk_surface_create_fn)
+    wlmtk_surface_t *wlmtk_surface_ptr)
 {
     wlmaker_layer_panel_t *layer_panel_ptr = logged_calloc(
         1, sizeof(wlmaker_layer_panel_t));
     if (NULL == layer_panel_ptr) return NULL;
     layer_panel_ptr->wlr_layer_surface_v1_ptr = wlr_layer_surface_v1_ptr;
     layer_panel_ptr->server_ptr = server_ptr;
+    layer_panel_ptr->wlmtk_surface_ptr = wlmtk_surface_ptr;
 
     wlmtk_panel_positioning_t pos = {};
-    if (!wlmtk_panel_init(
-            &layer_panel_ptr->super_panel, &pos, server_ptr->env_ptr)) {
+    if (!wlmtk_panel_init(&layer_panel_ptr->super_panel, &pos)) {
         _wlmaker_layer_panel_destroy(layer_panel_ptr);
         return NULL;
     }
     wlmtk_panel_extend(
         &layer_panel_ptr->super_panel,
-        &_wlmtk_layer_panel_vmt);
+        &_wlmaker_layer_panel_vmt);
+    wlmtk_element_extend(
+        wlmtk_panel_element(&layer_panel_ptr->super_panel),
+        &_wlmaker_layer_panel_element_vmt);
 
-    layer_panel_ptr->wlmtk_surface_ptr = wlmtk_surface_create_fn(
-        wlr_layer_surface_v1_ptr->surface,
-        server_ptr->env_ptr);
-    if (NULL == layer_panel_ptr->wlmtk_surface_ptr) {
-        _wlmaker_layer_panel_destroy(layer_panel_ptr);
-        return NULL;
-    }
     wlmtk_container_add_element_atop(
         &layer_panel_ptr->super_panel.super_container,
         NULL,
@@ -207,7 +230,6 @@ void _wlmaker_layer_panel_destroy(wlmaker_layer_panel_t *layer_panel_ptr)
 
     wlmtk_util_disconnect_listener(&layer_panel_ptr->new_popup_listener);
     wlmtk_util_disconnect_listener(&layer_panel_ptr->destroy_listener);
-
     wlmtk_util_disconnect_listener(&layer_panel_ptr->surface_commit_listener);
 
     wlmtk_util_disconnect_listener(&layer_panel_ptr->surface_unmap_listener);
@@ -222,9 +244,29 @@ void _wlmaker_layer_panel_destroy(wlmaker_layer_panel_t *layer_panel_ptr)
     }
 
     wlmtk_panel_fini(&layer_panel_ptr->super_panel);
+
+    if (layer_panel_ptr->wlr_layer_surface_v1_ptr) {
+        wlr_layer_surface_v1_destroy(layer_panel_ptr->wlr_layer_surface_v1_ptr);
+        layer_panel_ptr->wlr_layer_surface_v1_ptr = NULL;
+    }
     free(layer_panel_ptr);
 }
 
+/* ------------------------------------------------------------------------- */
+/**
+ * Implements @ref wlmtk_element_vmt_t::destroy, forwards to
+ * @ref _wlmaker_layer_panel_destroy.
+ *
+ * @param element_ptr
+ */
+void _wlmaker_layer_panel_element_destroy(
+    wlmtk_element_t *element_ptr)
+{
+    wlmaker_layer_panel_t *layer_panel_ptr = BS_CONTAINER_OF(
+        element_ptr, wlmaker_layer_panel_t, super_panel.super_container.super_element);
+    _wlmaker_layer_panel_destroy(layer_panel_ptr);
+}
+
 /* ------------------------------------------------------------------------- */
 /** @return Layer number, translated from protocol value. -1 on error. */
 wlmtk_workspace_layer_t _wlmaker_layer_from_zwlr_layer(
@@ -347,7 +389,9 @@ bool _wlmaker_layer_panel_apply_layer(
 
     if (NULL != layer_ptr) {
         wlmtk_layer_add_panel(
-            layer_ptr, &layer_panel_ptr->super_panel);
+            layer_ptr,
+            &layer_panel_ptr->super_panel,
+            layer_panel_ptr->wlr_layer_surface_v1_ptr->output);
     }
 
     return true;
@@ -476,6 +520,7 @@ void _wlmaker_layer_panel_handle_destroy(
     wlmaker_layer_panel_t *layer_panel_ptr = BS_CONTAINER_OF(
         listener_ptr, wlmaker_layer_panel_t, destroy_listener);
 
+    layer_panel_ptr->wlr_layer_surface_v1_ptr = NULL;
     _wlmaker_layer_panel_destroy(layer_panel_ptr);
 }
 
@@ -497,7 +542,7 @@ void _wlmaker_layer_panel_handle_new_popup(
 
     wlmaker_xdg_popup_t *popup_ptr = wlmaker_xdg_popup_create(
         wlr_xdg_popup_ptr,
-        layer_panel_ptr->server_ptr->env_ptr);
+        layer_panel_ptr->server_ptr->wlr_seat_ptr);
     if (NULL == popup_ptr) {
         wl_resource_post_error(
             wlr_xdg_popup_ptr->resource,
@@ -506,22 +551,24 @@ void _wlmaker_layer_panel_handle_new_popup(
         return;
     }
 
-    wlmtk_element_set_visible(
-        wlmtk_popup_element(&popup_ptr->super_popup), true);
     wlmtk_container_add_element(
         &layer_panel_ptr->super_panel.popup_container,
-        wlmtk_popup_element(&popup_ptr->super_popup));
+        wlmaker_xdg_popup_element(popup_ptr));
 }
 
 /* == Unit tests =========================================================== */
 
 static void test_create_destroy(bs_test_t *test_ptr);
 
-const bs_test_case_t wlmaker_layer_panel_test_cases[] = {
-    { 0, "create_destroy", test_create_destroy },
-    { 0, NULL, NULL }
+/** Unit test cases. */
+static const bs_test_case_t wlmaker_layer_panel_test_cases[] = {
+    { false, "create_destroy", test_create_destroy },
+    BS_TEST_CASE_SENTINEL()
 };
 
+const bs_test_set_t wlmaker_layer_panel_test_set = BS_TEST_SET(
+    true, "layer_panel", wlmaker_layer_panel_test_cases);
+
 /* ------------------------------------------------------------------------- */
 /** Exercises creation and teardown. */
 void test_create_destroy(bs_test_t *test_ptr)
@@ -529,17 +576,18 @@ void test_create_destroy(bs_test_t *test_ptr)
     struct wlr_layer_surface_v1 wlr_layer_surface_v1 = {};
     wlmaker_server_t server = {};
 
-    // Inject: wlmtk_surface_create - fake_surface_create.
+    wlmtk_surface_t *surface_ptr = wlmtk_surface_create(NULL, NULL);
+    BS_TEST_VERIFY_NEQ_OR_RETURN(test_ptr, NULL, surface_ptr);
 
     wl_signal_init(&wlr_layer_surface_v1.events.destroy);
     wl_signal_init(&wlr_layer_surface_v1.events.new_popup);
 
     wlmaker_layer_panel_t *layer_panel_ptr =
-        _wlmaker_layer_panel_create_injected(
+        _wlmaker_layer_panel_create_from_surface(
             &wlr_layer_surface_v1,
             &server,
-            wlmtk_fake_surface_create_inject);
-    BS_TEST_VERIFY_NEQ(test_ptr, NULL, layer_panel_ptr);
+            surface_ptr);
+    BS_TEST_VERIFY_NEQ_OR_RETURN(test_ptr, NULL, layer_panel_ptr);
 
     wl_signal_emit(&wlr_layer_surface_v1.events.destroy, NULL);
 }
