@@ -49,10 +49,8 @@
 /** State of the root menu. */
 struct _wlmaker_root_menu_t {
     /** Window. */
-    wlmtk_window_t            *window_ptr;
+    wlmtk_window_t           *window_ptr;
 
-    /** The root menu's window content base instance. */
-    wlmtk_content_t           content;
     /** The root menu base instance. */
     wlmtk_menu_t              *menu_ptr;
     /** Listener for @ref wlmtk_menu_events_t::open_changed. */
@@ -60,6 +58,11 @@ struct _wlmaker_root_menu_t {
     /** Listener for @ref wlmtk_menu_events_t::request_close. */
     struct wl_listener        menu_request_close_listener;
 
+    /** Listener for @ref wlmtk_window_events_t::request_close. */
+    struct wl_listener        window_request_close_listener;
+    /** Listener for @ref wlmtk_window_events_t::set_activated. */
+    struct wl_listener        window_set_activated_listener;
+
     /** Back-link to the server. */
     wlmaker_server_t          *server_ptr;
 };
@@ -80,11 +83,12 @@ typedef struct {
     struct wl_listener        menu_destroy_listener;
 } wlmaker_root_menu_generator_t;
 
-static void _wlmaker_root_menu_content_request_close(
-    wlmtk_content_t *content_ptr);
-static void _wlmaker_root_menu_content_set_activated(
-    wlmtk_content_t *content_ptr,
-    bool activated);
+static void _wlmaker_root_menu_handle_window_request_close(
+    struct wl_listener *listener_ptr,
+    void *data_ptr);
+static void _wlmaker_root_menu_handle_window_set_activated(
+    struct wl_listener *listener_ptr,
+    void *data_ptr);
 
 static void _wlmaker_root_menu_handle_menu_open_changed(
     struct wl_listener *listener_ptr,
@@ -136,19 +140,6 @@ static wlmtk_menu_item_t *_wlmaker_root_menu_create_disabled_item(
 
 /* == Data ================================================================= */
 
-/** Virtual method of the root menu's window content. */
-static const wlmtk_content_vmt_t _wlmaker_root_menu_content_vmt = {
-    .request_close = _wlmaker_root_menu_content_request_close,
-    .set_activated = _wlmaker_root_menu_content_set_activated,
-};
-
-/** Lookup paths for the root menu config file. */
-static const char *_wlmaker_root_menu_fname_ptrs[] = {
-    "~/.wlmaker-root-menu.plist",
-    "/usr/share/wlmaker/root-menu.plist",
-    NULL  // Sentinel.
-};
-
 /** Indicates to load the file specified in following argument. */
 static const char *_wlmaker_root_menu_statement_include = "IncludePlistMenu";
 
@@ -174,10 +165,11 @@ wlmaker_root_menu_t *wlmaker_root_menu_create(
     const wlmtk_menu_style_t *menu_style_ptr)
 {
     bspl_array_t *root_menu_array_ptr = bspl_array_from_object(
-        wlmaker_plist_load(
-            "root menu",
+        wlmaker_config_object_load(
+            server_ptr->files_ptr,
+            "root_menu",
             arg_root_menu_file_ptr,
-            _wlmaker_root_menu_fname_ptrs,
+            "RootMenu.plist",
             embedded_binary_root_menu_data,
             embedded_binary_root_menu_size));
     if (NULL == root_menu_array_ptr) return NULL;
@@ -222,48 +214,24 @@ wlmaker_root_menu_t *wlmaker_root_menu_create(
         &root_menu_ptr->menu_request_close_listener,
         _wlmaker_root_menu_handle_request_close);
 
-    // FIXME - really terrible hack.
-    wlmtk_pane_t *pane_ptr = wlmtk_menu_pane(root_menu_ptr->menu_ptr);
-    struct wlr_box box = wlmtk_element_get_dimensions_box(
-        wlmtk_menu_element(root_menu_ptr->menu_ptr));
-    wlmtk_container_remove_element(
-        &pane_ptr->super_container,
-        pane_ptr->element_ptr);
-    if (!wlmtk_content_init(
-            &root_menu_ptr->content,
-            pane_ptr->element_ptr)) {
-        wlmaker_root_menu_destroy(root_menu_ptr);
-        bspl_array_unref(root_menu_array_ptr);
-        return NULL;
-    }
-    wlmtk_container_remove_element(
-        &pane_ptr->super_container,
-        &pane_ptr->popup_container.super_element);
-    wlmtk_container_add_element(
-        &root_menu_ptr->content.popup_container,
-        &pane_ptr->popup_container.super_element);
-
-    wlmtk_content_extend(
-        &root_menu_ptr->content,
-        &_wlmaker_root_menu_content_vmt);
-    // TODO(kaeser@gubbe.ch): Should not be required. Also, the sequence
-    // of set_server_side_decorated and set_attributes is brittle.
-    wlmtk_content_commit(
-        &root_menu_ptr->content,
-        box.width,
-        box.height,
-        0);
-
     root_menu_ptr->window_ptr = wlmtk_window_create(
-        &root_menu_ptr->content,
+        wlmtk_menu_element(root_menu_ptr->menu_ptr),
         window_style_ptr,
-        menu_style_ptr,
-        server_ptr->wlr_seat_ptr);
+        menu_style_ptr);
     if (NULL == root_menu_ptr->window_ptr) {
         wlmaker_root_menu_destroy(root_menu_ptr);
         bspl_array_unref(root_menu_array_ptr);
         return NULL;
     }
+    wlmtk_util_connect_listener_signal(
+        &wlmtk_window_events(root_menu_ptr->window_ptr)->request_close,
+        &root_menu_ptr->window_request_close_listener,
+        _wlmaker_root_menu_handle_window_request_close);
+    wlmtk_util_connect_listener_signal(
+        &wlmtk_window_events(root_menu_ptr->window_ptr)->set_activated,
+        &root_menu_ptr->window_set_activated_listener,
+        _wlmaker_root_menu_handle_window_set_activated);
+
     wlmtk_window_set_title(
         root_menu_ptr->window_ptr,
         bspl_array_string_value_at(root_menu_array_ptr, 0));
@@ -288,29 +256,18 @@ void wlmaker_root_menu_destroy(wlmaker_root_menu_t *root_menu_ptr)
             root_menu_ptr->window_ptr);
         if (NULL != workspace_ptr) {
             wlmtk_workspace_unmap_window(workspace_ptr,
-                                         root_menu_ptr->window_ptr);
+                                          root_menu_ptr->window_ptr);
         }
 
+        wlmtk_util_disconnect_listener(
+            &root_menu_ptr->window_request_close_listener);
+        wlmtk_util_disconnect_listener(
+            &root_menu_ptr->window_set_activated_listener);
+
         wlmtk_window_destroy(root_menu_ptr->window_ptr);
         root_menu_ptr->window_ptr = NULL;
     }
 
-    if (NULL != root_menu_ptr->menu_ptr) {
-        wlmtk_content_set_element(&root_menu_ptr->content, NULL);
-        wlmtk_pane_t *pane_ptr = wlmtk_menu_pane(root_menu_ptr->menu_ptr);
-        wlmtk_container_add_element(
-            &pane_ptr->super_container,
-            pane_ptr->element_ptr);
-
-        wlmtk_container_remove_element(
-            &root_menu_ptr->content.popup_container,
-            &pane_ptr->popup_container.super_element);
-        wlmtk_container_add_element(
-            &pane_ptr->super_container,
-            &pane_ptr->popup_container.super_element);
-    }
-
-    wlmtk_content_fini(&root_menu_ptr->content);
     if (NULL != root_menu_ptr->menu_ptr) {
         wlmtk_util_disconnect_listener(
             &root_menu_ptr->menu_request_close_listener);
@@ -337,32 +294,36 @@ wlmtk_menu_t *wlmaker_root_menu_menu(wlmaker_root_menu_t *root_menu_ptr)
 /* == Local (static) methods =============================================== */
 
 /* ------------------------------------------------------------------------- */
-/** Implements @ref wlmtk_content_vmt_t::request_close. Closes root menu. */
-void _wlmaker_root_menu_content_request_close(
-    wlmtk_content_t *content_ptr)
+/** Handles when window close button is pressed: Hides the menu. */
+void _wlmaker_root_menu_handle_window_request_close(
+    struct wl_listener *listener_ptr,
+    __UNUSED__ void *data_ptr)
 {
     wlmaker_root_menu_t *root_menu_ptr = BS_CONTAINER_OF(
-        content_ptr, wlmaker_root_menu_t, content);
-
+        listener_ptr, wlmaker_root_menu_t, window_request_close_listener);
     wlmtk_menu_set_open(root_menu_ptr->menu_ptr, false);
+
 }
 
 /* ------------------------------------------------------------------------- */
-/** Imlements @ref wlmtk_content_vmt_t::set_activated. Gets keyboard focus. */
-void _wlmaker_root_menu_content_set_activated(
-    wlmtk_content_t *content_ptr,
-    bool activated)
+/** Handles when the menu is activated: Get keyboard focus. */
+void _wlmaker_root_menu_handle_window_set_activated(
+    struct wl_listener *listener_ptr,
+    __UNUSED__ void *data_ptr)
 {
     wlmaker_root_menu_t *root_menu_ptr = BS_CONTAINER_OF(
-        content_ptr, wlmaker_root_menu_t, content);
+        listener_ptr, wlmaker_root_menu_t, window_set_activated_listener);
 
-    wlmtk_element_t *e = wlmtk_menu_pane(root_menu_ptr->menu_ptr)->element_ptr;
+    wlmtk_element_t *e = wlmtk_menu_element(root_menu_ptr->menu_ptr);
     if (NULL != e->parent_container_ptr) {
         wlmtk_container_set_keyboard_focus_element(
-            e->parent_container_ptr, e, activated);
+            e->parent_container_ptr,
+            e,
+            wlmtk_window_is_activated(root_menu_ptr->window_ptr));
     }
 }
 
+
 /* ------------------------------------------------------------------------- */
 /** Handles @ref wlmtk_menu_events_t::open_changed. Unmaps window on close. */
 void _wlmaker_root_menu_handle_menu_open_changed(
@@ -382,14 +343,10 @@ void _wlmaker_root_menu_handle_menu_open_changed(
         if (WLMTK_MENU_MODE_RIGHTCLICK ==
             wlmtk_menu_get_mode(root_menu_ptr->menu_ptr)) {
             properties |= WLMTK_WINDOW_PROPERTY_RIGHTCLICK;
-
-            // TODO(kaeser@gubbe.ch): Also undo, with that really terrible
-            // hack of hacking the pane into the content.
-            wlmtk_element_t *content_element_ptr =
-                wlmtk_content_element(&root_menu_ptr->content);
             wlmtk_container_pointer_grab(
-                content_element_ptr->parent_container_ptr,
-                content_element_ptr);
+                wlmtk_menu_element(
+                    root_menu_ptr->menu_ptr)->parent_container_ptr,
+                wlmtk_menu_element(root_menu_ptr->menu_ptr));
         } else {
             properties |= WLMTK_WINDOW_PROPERTY_CLOSABLE;
         }
@@ -884,12 +841,15 @@ wlmtk_menu_item_t *_wlmaker_root_menu_create_disabled_item(
 static void test_default_menu(bs_test_t *test_ptr);
 static void test_generated_menu(bs_test_t *test_ptr);
 
-const bs_test_case_t wlmaker_root_menu_test_cases[] = {
-    { 1, "default_menu", test_default_menu },
-    { 1, "generated_menu", test_generated_menu },
-    { 0, NULL, NULL }
+/** Unit test cases. */
+static const bs_test_case_t wlmaker_root_menu_test_cases[] = {
+    { true, "default_menu", test_default_menu },
+    { true, "generated_menu", test_generated_menu },
+    BS_TEST_CASE_SENTINEL()
 };
 
+const bs_test_set_t wlmaker_root_menu_test_set = BS_TEST_SET(
+    true, "root_menu", wlmaker_root_menu_test_cases);
 
 /* ------------------------------------------------------------------------- */
 /** Verifies that the compiled-in configuration translates into a menu. */
@@ -962,13 +922,14 @@ void test_generated_menu(bs_test_t *test_ptr)
     // Exercise & verify that a menu can be generated from output of
     // Window Maker's `wmmenugen` command. This is conditional of that binary
     // being present on the host.
+    // Note: RootMenuDebian.plist is generated, it in WLMAKER_BINARY_DIR.
 #ifndef WLMAKER_BINARY_DIR
 #error "Missing definition of WLMAKER_BINARY_DIR!"
 #endif
 #if defined(WLMAKER_WMMENUGEN_PATH)
     root_menu_ptr = wlmaker_root_menu_create(
         &server,
-        WLMAKER_BINARY_DIR "/etc/root-menu-debian.plist",
+        WLMAKER_BINARY_DIR "/etc/RootMenuDebian.plist",
         &window_style, &menu_style);
     BS_TEST_VERIFY_NEQ_OR_RETURN(test_ptr, NULL, root_menu_ptr);
     menu_ptr = wlmaker_root_menu_menu(root_menu_ptr);
