@@ -30,7 +30,9 @@
 #include <xkbcommon/xkbcommon-keysyms.h>
 #define WLR_USE_UNSTABLE
 #include <wlr/backend.h>
+#include <wlr/types/wlr_viewporter.h>
 #include <wlr/types/wlr_cursor.h>
+#include <wlr/types/wlr_fractional_scale_v1.h>
 #include <wlr/types/wlr_data_device.h>
 #include <wlr/types/wlr_input_device.h>
 #include <wlr/types/wlr_output_layout.h>
@@ -102,11 +104,13 @@ const uint32_t wlmaker_modifier_default_mask = (
 /* ------------------------------------------------------------------------- */
 wlmaker_server_t *wlmaker_server_create(
     bspl_dict_t *config_dict_ptr,
+    wlmaker_files_t *files_ptr,
     const wlmaker_server_options_t *options_ptr)
 {
     wlmaker_server_t *server_ptr = logged_calloc(1, sizeof(wlmaker_server_t));
     if (NULL == server_ptr) return NULL;
     server_ptr->options_ptr = options_ptr;
+    server_ptr->files_ptr = files_ptr;
 
     server_ptr->config_dict_ptr = bspl_dict_ref(config_dict_ptr);
     if (NULL == server_ptr->config_dict_ptr) {
@@ -135,6 +139,22 @@ wlmaker_server_t *wlmaker_server_create(
         return NULL;
     }
 
+    server_ptr->wlr_viewporter_ptr = wlr_viewporter_create(
+        server_ptr->wl_display_ptr);
+    if (NULL == server_ptr->wlr_viewporter_ptr) {
+        bs_log(BS_ERROR, "Failed wlr_viewporter_create()");
+        wlmaker_server_destroy(server_ptr);
+        return NULL;
+    }
+
+    server_ptr->wlr_fractional_scale_manager_ptr =
+        wlr_fractional_scale_manager_v1_create(server_ptr->wl_display_ptr, 1);
+    if (NULL == server_ptr->wlr_fractional_scale_manager_ptr) {
+        bs_log(BS_ERROR, "Failed wlr_fractional_scale_manager_v1_create()");
+        wlmaker_server_destroy(server_ptr);
+        return NULL;
+    }
+
     // Configure the seat, which is the potential set of input devices operated
     // by one user at a computer's "seat".
     server_ptr->wlr_seat_ptr = wlr_seat_create(
@@ -161,13 +181,21 @@ wlmaker_server_t *wlmaker_server_create(
         return NULL;
     }
 
+    char *f = wlmaker_files_xdg_config_fname(
+        server_ptr->files_ptr, "OutputState.plist");
+    if (NULL == f) {
+        wlmaker_server_destroy(server_ptr);
+        return NULL;
+    }
     server_ptr->backend_ptr = wlmbe_backend_create(
         server_ptr->wl_display_ptr,
         server_ptr->wlr_scene_ptr,
         server_ptr->wlr_output_layout_ptr,
         server_ptr->options_ptr->width,
         server_ptr->options_ptr->height,
-        server_ptr->config_dict_ptr);
+        server_ptr->config_dict_ptr,
+        f);
+    free(f);
     if (NULL == server_ptr->backend_ptr) {
         bs_log(BS_ERROR, "Failed wlmbe_backend_create()");
         wlmaker_server_destroy(server_ptr);
@@ -235,9 +263,14 @@ wlmaker_server_t *wlmaker_server_create(
     }
 
     server_ptr->xdg_decoration_manager_ptr =
-        wlmaker_xdg_decoration_manager_create(server_ptr);
+        wlmaker_xdg_decoration_manager_create(
+            server_ptr->wl_display_ptr,
+            server_ptr->config_dict_ptr);
     if (NULL == server_ptr->xdg_decoration_manager_ptr) {
-        bs_log(BS_ERROR, "Failed wlmaker_xdg_decoration_manager_create()");
+        bs_log(BS_ERROR,
+               "Failed wlmaker_xdg_decoration_manager_create(%p, %p)",
+               server_ptr->wl_display_ptr,
+               server_ptr->config_dict_ptr);
         wlmaker_server_destroy(server_ptr);
         return NULL;
     }
@@ -256,6 +289,16 @@ wlmaker_server_t *wlmaker_server_create(
         return NULL;
     }
 
+    server_ptr->input_observation_manager_ptr =
+        wlmaker_input_observation_manager_create(
+            server_ptr->wl_display_ptr,
+            server_ptr->wlr_seat_ptr,
+            server_ptr->cursor_ptr->wlr_cursor_ptr);
+    if (NULL == server_ptr->input_observation_manager_ptr) {
+        wlmaker_server_destroy(server_ptr);
+        return NULL;
+    }
+
     if (server_ptr->options_ptr->start_xwayland) {
         server_ptr->xwl_ptr = wlmaker_xwl_create(server_ptr);
         if (NULL == server_ptr->xwl_ptr) {
@@ -330,6 +373,12 @@ void wlmaker_server_destroy(wlmaker_server_t *server_ptr)
         server_ptr->xwl_ptr = NULL;
     }
 
+    if (NULL != server_ptr->input_observation_manager_ptr) {
+        wlmaker_input_observation_manager_destroy(
+            server_ptr->input_observation_manager_ptr);
+        server_ptr->input_observation_manager_ptr = NULL;
+    }
+
     if (NULL != server_ptr->icon_manager_ptr) {
         wlmaker_icon_manager_destroy(server_ptr->icon_manager_ptr);
         server_ptr->icon_manager_ptr = NULL;
@@ -393,6 +442,11 @@ void wlmaker_server_destroy(wlmaker_server_t *server_ptr)
         server_ptr->config_dict_ptr = NULL;
     }
 
+    if (NULL != server_ptr->files_ptr) {
+        wlmaker_files_destroy(server_ptr->files_ptr);
+        server_ptr->files_ptr = NULL;
+    }
+
     free(server_ptr);
 }
 
@@ -627,7 +681,8 @@ void _wlmaker_server_unclaimed_button_event_handler(
         wlmtk_workspace_map_window(
             wlmtk_root_get_current_workspace(server_ptr->root_ptr),
             wlmaker_root_menu_window(server_ptr->root_menu_ptr));
-        wlmtk_window_set_position(
+        wlmtk_workspace_set_window_position(
+            wlmtk_root_get_current_workspace(server_ptr->root_ptr),
             wlmaker_root_menu_window(server_ptr->root_menu_ptr),
             server_ptr->cursor_ptr->wlr_cursor_ptr->x,
             server_ptr->cursor_ptr->wlr_cursor_ptr->y);
@@ -649,10 +704,13 @@ static void test_bind(bs_test_t *test_ptr);
 
 /** Test cases for the server. */
 const bs_test_case_t          wlmaker_server_test_cases[] = {
-    { 1, "bind", test_bind },
-    { 0, NULL, NULL }
+    { true, "bind", test_bind },
+    BS_TEST_CASE_SENTINEL()
 };
 
+const bs_test_set_t wlmaker_server_test_set = BS_TEST_SET(
+    true, "server", wlmaker_server_test_cases);
+
 /** Test helper: Callback for a keybinding. */
 bool test_binding_callback(
     __UNUSED__ const wlmaker_key_combo_t *key_combo_ptr) {
