@@ -32,11 +32,9 @@
 #include <wlr/types/wlr_seat.h>
 #undef WLR_USE_UNSTABLE
 
-struct _wlmaker_server_t;
 /** A handle for a wlmaker server. */
 typedef struct _wlmaker_server_t wlmaker_server_t;
 
-struct _wlmaker_key_combo_t;
 /** A key combination. */
 typedef struct _wlmaker_key_combo_t wlmaker_key_combo_t;
 /** Handle for a key binding. */
@@ -55,7 +53,9 @@ typedef bool (*wlmaker_keybinding_callback_t)(const wlmaker_key_combo_t *kc);
 #include "config.h"
 #include "corner.h"  // IWYU pragma: keep
 #include "cursor.h"  // IWYU pragma: keep
+#include "files.h"
 #include "icon_manager.h"  // IWYU pragma: keep
+#include "input_observation.h"
 #include "idle.h"  // IWYU pragma: keep
 #include "layer_shell.h"  // IWYU pragma: keep
 #include "lock_mgr.h"  // IWYU pragma: keep
@@ -78,12 +78,16 @@ typedef struct {
     uint32_t                  width;
     /** Desired output height, for windowed mode. 0 for no preference. */
     uint32_t                  height;
+    /** Whether to include 'Logo' to modifiers. */
+    bool                      bind_with_logo;
 } wlmaker_server_options_t;
 
 /** State of the Wayland server. */
 struct _wlmaker_server_t {
+    /** Files module handle. */
+    wlmaker_files_t           *files_ptr;
     /** Configuration dictionnary. */
-    bspl_dict_t             *config_dict_ptr;
+    bspl_dict_t               *config_dict_ptr;
     /** Copy of the options. */
     const wlmaker_server_options_t *options_ptr;
 
@@ -97,6 +101,10 @@ struct _wlmaker_server_t {
     /** Idle monitor. */
     wlmaker_idle_monitor_t    *idle_monitor_ptr;
 
+    /** WLR viewporter. */
+    struct wlr_viewporter     *wlr_viewporter_ptr;
+    /** Fractional scale manager. */
+    struct wlr_fractional_scale_manager_v1 *wlr_fractional_scale_manager_ptr;
     /** wlroots seat. */
     struct wlr_seat           *wlr_seat_ptr;
     /** The scene graph API. */
@@ -124,6 +132,8 @@ struct _wlmaker_server_t {
     wlmbe_backend_t           *backend_ptr;
     /** Icon manager. */
     wlmaker_icon_manager_t    *icon_manager_ptr;
+    /** Input observation. */
+    wlmaker_input_observation_manager_t *input_observation_manager_ptr;
     /**
      * XWayland interface. Will be set only if compiled with XWayland, through
      * WLMAKER_HAVE_XWAYLAND defined.
@@ -193,6 +203,7 @@ struct _wlmaker_key_combo_t {
  *
  * @param config_dict_ptr     Configuration, as dictionary object. The server
  *                            will keep a reference on it until destroyed.
+ * @param files_ptr
  * @param options_ptr         Options for the server. The server expects the
  *                            pointed area to outlive the server.
  *
@@ -201,6 +212,7 @@ struct _wlmaker_key_combo_t {
  */
 wlmaker_server_t *wlmaker_server_create(
     bspl_dict_t *config_dict_ptr,
+    wlmaker_files_t *files_ptr,
     const wlmaker_server_options_t *options_ptr);
 
 /**
@@ -275,8 +287,8 @@ struct wlr_output *wlmaker_server_get_output_at_cursor(
 /** All modifiers to use by default. */
 extern const uint32_t wlmaker_modifier_default_mask;
 
-/** Unit test cases. */
-extern const bs_test_case_t   wlmaker_server_test_cases[];
+/** Unit test set. */
+extern const bs_test_set_t   wlmaker_server_test_set;
 
 #ifdef __cplusplus
 }  // extern "C"
