@@ -19,10 +19,14 @@
  */
 
 #include "panel.h"
-#include "test.h"
 
+#include <libbase/libbase.h>
+#include <stdlib.h>
+#include <string.h>
 #include <wlr/util/edges.h>
 
+#include "test.h"  // IWYU pragma: keep
+
 /* == Declarations ========================================================= */
 
 /* == Exported methods ===================================================== */
@@ -30,17 +34,16 @@
 /* ------------------------------------------------------------------------- */
 bool wlmtk_panel_init(
     wlmtk_panel_t *panel_ptr,
-    const wlmtk_panel_positioning_t *positioning_ptr,
-    wlmtk_env_t *env_ptr)
+    const wlmtk_panel_positioning_t *positioning_ptr)
 {
-    memset(panel_ptr, 0, sizeof(wlmtk_panel_t));
-    if (!wlmtk_container_init(&panel_ptr->super_container, env_ptr)) {
+    *panel_ptr = (wlmtk_panel_t){};
+    if (!wlmtk_container_init(&panel_ptr->super_container)) {
         wlmtk_panel_fini(panel_ptr);
         return false;
     }
     panel_ptr->positioning = *positioning_ptr;
 
-    if (!wlmtk_container_init(&panel_ptr->popup_container, env_ptr)) {
+    if (!wlmtk_container_init(&panel_ptr->popup_container)) {
         wlmtk_panel_fini(panel_ptr);
         return false;
     }
@@ -114,6 +117,20 @@ wlmtk_layer_t *wlmtk_panel_get_layer(wlmtk_panel_t *panel_ptr)
     return panel_ptr->layer_ptr;
 }
 
+/* ------------------------------------------------------------------------- */
+void wlmtk_panel_set_layer_output(
+    wlmtk_panel_t *panel_ptr,
+    wlmtk_layer_output_t *layer_output_ptr)
+{
+    panel_ptr->layer_output_ptr = layer_output_ptr;
+}
+
+/* ------------------------------------------------------------------------- */
+wlmtk_layer_output_t *wlmtk_panel_get_layer_output(wlmtk_panel_t *panel_ptr)
+{
+    return panel_ptr->layer_output_ptr;
+}
+
 /* ------------------------------------------------------------------------- */
 void wlmtk_panel_commit(
     wlmtk_panel_t *panel_ptr,
@@ -132,8 +149,8 @@ void wlmtk_panel_commit(
 
     panel_ptr->positioning = *positioning_ptr;
 
-    if (NULL != panel_ptr->layer_ptr) {
-        wlmtk_layer_reconfigure(panel_ptr->layer_ptr);
+    if (NULL != panel_ptr->layer_output_ptr) {
+        wlmtk_layer_output_reconfigure(panel_ptr->layer_output_ptr);
     }
 }
 
@@ -223,11 +240,17 @@ struct wlr_box wlmtk_panel_compute_dimensions(
 
 /* == Local (static) methods =============================================== */
 
+static void _wlmtk_fake_panel_element_destroy(
+    wlmtk_element_t *element_ptr);
 static uint32_t _wlmtk_fake_panel_request_size(
     wlmtk_panel_t *panel_ptr,
     int width,
     int height);
 
+/** Virtual methods of the fake panel's element superclass. */
+static const wlmtk_element_vmt_t _wlmtk_fake_panel_element_vmt = {
+    .destroy = _wlmtk_fake_panel_element_destroy
+};
 /** Virtual methods of the fake panel. */
 static const wlmtk_panel_vmt_t _wlmtk_fake_panel_vmt = {
     .request_size = _wlmtk_fake_panel_request_size
@@ -241,10 +264,13 @@ wlmtk_fake_panel_t *wlmtk_fake_panel_create(
         1, sizeof(wlmtk_fake_panel_t));
     if (NULL == fake_panel_ptr) return NULL;
 
-    if (!wlmtk_panel_init(&fake_panel_ptr->panel, positioning_ptr, NULL)) {
+    if (!wlmtk_panel_init(&fake_panel_ptr->panel, positioning_ptr)) {
         wlmtk_fake_panel_destroy(fake_panel_ptr);
         return NULL;
     }
+    wlmtk_element_extend(
+        &fake_panel_ptr->panel.super_container.super_element,
+        &_wlmtk_fake_panel_element_vmt);
     wlmtk_panel_extend(&fake_panel_ptr->panel, &_wlmtk_fake_panel_vmt);
 
     return fake_panel_ptr;
@@ -257,7 +283,15 @@ void wlmtk_fake_panel_destroy(wlmtk_fake_panel_t *fake_panel_ptr)
     free(fake_panel_ptr);
 }
 
-/* ------------------------------------------------------------------------- */
+/** Implements @ref wlmtk_element_vmt_t::destroy for the fake panel. */
+void _wlmtk_fake_panel_element_destroy(
+    wlmtk_element_t *element_ptr)
+{
+    wlmtk_fake_panel_t *fake_panel_ptr = BS_CONTAINER_OF(
+        element_ptr, wlmtk_fake_panel_t, panel.super_container.super_element);
+    wlmtk_fake_panel_destroy(fake_panel_ptr);
+}
+
 /** Fake implementation of @ref wlmtk_panel_vmt_t::request_size. */
 uint32_t _wlmtk_fake_panel_request_size(
     wlmtk_panel_t *panel_ptr,
@@ -292,7 +326,7 @@ void test_init_fini(bs_test_t *test_ptr)
     wlmtk_panel_t p;
     wlmtk_panel_positioning_t pos = {};
 
-    BS_TEST_VERIFY_TRUE(test_ptr, wlmtk_panel_init(&p, &pos, NULL));
+    BS_TEST_VERIFY_TRUE(test_ptr, wlmtk_panel_init(&p, &pos));
 
     bs_dllist_node_t *dlnode_ptr = wlmtk_dlnode_from_panel(&p);
     BS_TEST_VERIFY_EQ(test_ptr, &p.dlnode, dlnode_ptr);
