@@ -20,12 +20,26 @@
 
 #include "root.h"
 
-#include <wlr/version.h>
+#include <libbase/libbase.h>
+#include <stdlib.h>
+#include <wayland-server-protocol.h>
 #define WLR_USE_UNSTABLE
+#include <wlr/types/wlr_output.h>
 #include <wlr/types/wlr_output_layout.h>
 #include <wlr/types/wlr_scene.h>
 #undef WLR_USE_UNSTABLE
 
+#include "container.h"
+#include "input.h"
+#include "rectangle.h"
+#include "test.h"  // IWYU pragma: keep
+#include "tile.h"
+#include "util.h"
+#include "window.h"
+#include "workspace.h"
+
+struct wlr_keyboard_key_event;
+
 /* == Declarations ========================================================= */
 
 /** State of the root element. */
@@ -42,8 +56,11 @@ struct _wlmtk_root_t {
 
     /** Whether the root is currently locked. */
     bool                      locked;
-    /** Reference to the lock, see @ref wlmtk_root_lock. */
-    wlmtk_lock_t               *lock_ptr;
+    /**
+     * The lock's element. Shown on top of
+     * @ref wlmtk_root_t::curtain_rectangle_ptr.
+     */
+    wlmtk_element_t           *lock_element_ptr;
 
     /** Curtain element: Permit dimming or hiding everything. */
     wlmtk_rectangle_t         *curtain_rectangle_ptr;
@@ -52,14 +69,21 @@ struct _wlmtk_root_t {
     bs_dllist_t               workspaces;
     /** Currently-active workspace. */
     wlmtk_workspace_t         *current_workspace_ptr;
+
+    /** Listener for wlr_output_layout::events.change. */
+    struct wl_listener        output_layout_change_listener;
+
+    // Elements below not owned by wlmtk_root_t.
+    /** wlroots output layout. */
+    struct wlr_output_layout  *wlr_output_layout_ptr;
+
+    /** Last recorded pointer movement. */
+    wlmtk_pointer_motion_event_t mev;
 };
 
 static void _wlmtk_root_switch_to_workspace(
     wlmtk_root_t *root_ptr,
     wlmtk_workspace_t *workspace_ptr);
-static void _wlmtk_root_set_workspace_extents(
-    bs_dllist_node_t *dlnode_ptr,
-    void *ud_ptr);
 static void _wlmtk_root_enumerate_workspaces(
     bs_dllist_node_t *dlnode_ptr,
     void *ud_ptr);
@@ -67,10 +91,6 @@ static void _wlmtk_root_destroy_workspace(
     bs_dllist_node_t *dlnode_ptr,
     void *ud_ptr);
 
-static bool _wlmtk_root_element_pointer_motion(
-    wlmtk_element_t *element_ptr,
-    double x, double y,
-    uint32_t time_msec);
 static bool _wlmtk_root_element_pointer_button(
     wlmtk_element_t *element_ptr,
     const wlmtk_button_event_t *button_event_ptr);
@@ -79,14 +99,14 @@ static bool _wlmtk_root_element_pointer_axis(
     struct wlr_pointer_axis_event *wlr_pointer_axis_event_ptr);
 static bool _wlmtk_root_element_keyboard_event(
     wlmtk_element_t *element_ptr,
-    struct wlr_keyboard_key_event *wlr_keyboard_key_event_ptr,
-    const xkb_keysym_t *key_syms,
-    size_t key_syms_count,
-    uint32_t modifiers);
+    struct wlr_keyboard_key_event *wlr_keyboard_key_event_ptr);
+
+static void _wlmtk_root_handle_output_layout_change(
+    struct wl_listener *listener_ptr,
+    void *data_ptr);
 
 /** Virtual method table for the container's super class: Element. */
 static const wlmtk_element_vmt_t _wlmtk_root_element_vmt = {
-    .pointer_motion = _wlmtk_root_element_pointer_motion,
     .pointer_button = _wlmtk_root_element_pointer_button,
     .pointer_axis = _wlmtk_root_element_pointer_axis,
     .keyboard_event = _wlmtk_root_element_keyboard_event,
@@ -97,17 +117,24 @@ static const wlmtk_element_vmt_t _wlmtk_root_element_vmt = {
 /* ------------------------------------------------------------------------- */
 wlmtk_root_t *wlmtk_root_create(
     struct wlr_scene *wlr_scene_ptr,
-    wlmtk_env_t *env_ptr)
+    struct wlr_output_layout *wlr_output_layout_ptr)
 {
     wlmtk_root_t *root_ptr = logged_calloc(1, sizeof(wlmtk_root_t));
     if (NULL == root_ptr) return NULL;
+    root_ptr->wlr_output_layout_ptr = wlr_output_layout_ptr;
 
-    if (!wlmtk_container_init_attached(
-            &root_ptr->container,
-            env_ptr,
-            &wlr_scene_ptr->tree)) {
-        wlmtk_root_destroy(root_ptr);
-        return NULL;
+    if (NULL != wlr_scene_ptr) {
+        if (!wlmtk_container_init_attached(
+                &root_ptr->container,
+                &wlr_scene_ptr->tree)) {
+            wlmtk_root_destroy(root_ptr);
+            return NULL;
+        }
+    } else {
+        if (!wlmtk_container_init(&root_ptr->container)) {
+            wlmtk_root_destroy(root_ptr);
+            return NULL;
+        }
     }
     wlmtk_element_set_visible(&root_ptr->container.super_element, true);
     root_ptr->orig_super_element_vmt = wlmtk_element_extend(
@@ -115,7 +142,7 @@ wlmtk_root_t *wlmtk_root_create(
         &_wlmtk_root_element_vmt);
 
     root_ptr->curtain_rectangle_ptr = wlmtk_rectangle_create(
-        env_ptr, root_ptr->extents.width, root_ptr->extents.height, 0xff000020);
+        root_ptr->extents.width, root_ptr->extents.height, 0xff000020);
     if (NULL == root_ptr->curtain_rectangle_ptr) {
         wlmtk_root_destroy(root_ptr);
         return NULL;
@@ -124,6 +151,14 @@ wlmtk_root_t *wlmtk_root_create(
         &root_ptr->container,
         wlmtk_rectangle_element(root_ptr->curtain_rectangle_ptr));
 
+    wlmtk_util_connect_listener_signal(
+        &wlr_output_layout_ptr->events.change,
+        &root_ptr->output_layout_change_listener,
+        _wlmtk_root_handle_output_layout_change);
+    _wlmtk_root_handle_output_layout_change(
+        &root_ptr->output_layout_change_listener,
+        wlr_output_layout_ptr);
+
     wl_signal_init(&root_ptr->events.workspace_changed);
     wl_signal_init(&root_ptr->events.unlock_event);
     wl_signal_init(&root_ptr->events.window_mapped);
@@ -135,6 +170,9 @@ wlmtk_root_t *wlmtk_root_create(
 /* ------------------------------------------------------------------------- */
 void wlmtk_root_destroy(wlmtk_root_t *root_ptr)
 {
+    wlmtk_util_disconnect_listener(
+        &root_ptr->output_layout_change_listener);
+
     bs_dllist_for_each(
         &root_ptr->workspaces,
         _wlmtk_root_destroy_workspace,
@@ -160,33 +198,19 @@ wlmtk_root_events_t *wlmtk_root_events(wlmtk_root_t *root_ptr)
     return &root_ptr->events;
 }
 
-/* ------------------------------------------------------------------------- */
-void wlmtk_root_set_extents(
-    wlmtk_root_t *root_ptr,
-    const struct wlr_box *extents_ptr)
-{
-    root_ptr->extents = *extents_ptr;
-
-    wlmtk_rectangle_set_size(
-        root_ptr->curtain_rectangle_ptr,
-        root_ptr->extents.width,
-        root_ptr->extents.height);
-
-    bs_dllist_for_each(
-        &root_ptr->workspaces, _wlmtk_root_set_workspace_extents,
-        &root_ptr->extents);
-}
-
 /* ------------------------------------------------------------------------- */
 bool wlmtk_root_pointer_motion(
     wlmtk_root_t *root_ptr,
     double x,
     double y,
-    uint32_t time_msec)
+    uint32_t time_msec,
+    wlmtk_pointer_t *pointer_ptr)
 {
+    root_ptr->mev = (wlmtk_pointer_motion_event_t){
+        .x = x, .y = y, .time_msec = time_msec, .pointer_ptr = pointer_ptr
+    };
     return wlmtk_element_pointer_motion(
-        &root_ptr->container.super_element, x, y, time_msec);
-
+        &root_ptr->container.super_element, &root_ptr->mev);
 }
 
 /* ------------------------------------------------------------------------- */
@@ -194,7 +218,8 @@ bool wlmtk_root_pointer_motion(
 // different buttons apart, and there's currently no test associated.
 bool wlmtk_root_pointer_button(
     wlmtk_root_t *root_ptr,
-    const struct wlr_pointer_button_event *event_ptr)
+    const struct wlr_pointer_button_event *event_ptr,
+    uint32_t modifiers)
 {
     wlmtk_button_event_t event;
     bool rv;
@@ -202,28 +227,30 @@ bool wlmtk_root_pointer_button(
     // Guard clause: nothing to pass on if no element has the focus.
     event.button = event_ptr->button;
     event.time_msec = event_ptr->time_msec;
+    event.keyboard_modifiers = modifiers;
     switch (event_ptr->state) {
-#if WLR_VERSION_NUM >= (18 << 8)
     case WL_POINTER_BUTTON_STATE_PRESSED:
-#else // WLR_VERSION_NUM >= (18 << 8)
-    case WLR_BUTTON_PRESSED:
-#endif // WLR_VERSION_NUM >= (18 << 8)
         event.type = WLMTK_BUTTON_DOWN;
         rv = wlmtk_element_pointer_button(
             &root_ptr->container.super_element, &event);
         break;
 
-#if WLR_VERSION_NUM >= (18 << 8)
     case WL_POINTER_BUTTON_STATE_RELEASED:
-#else // WLR_VERSION_NUM >= (18 << 8)
-    case WLR_BUTTON_RELEASED:
-#endif // WLR_VERSION_NUM >= (18 << 8)
         event.type = WLMTK_BUTTON_UP;
         wlmtk_element_pointer_button(
             &root_ptr->container.super_element, &event);
         event.type = WLMTK_BUTTON_CLICK;
         rv = wlmtk_element_pointer_button(
             &root_ptr->container.super_element, &event);
+
+        // Hack: Fix lost focus when a menu releases the pointer grab. This
+        // should not be needed here, but needs a better means of updating
+        // the pointer focus.
+        // TODO(kaeser@gubbe.ch): Have a test for this with a window + menu,
+        // and ensure focus computation is done well there.
+        wlmtk_element_pointer_motion(
+            &root_ptr->container.super_element,
+            &root_ptr->mev);
         break;
 
     default:
@@ -261,10 +288,7 @@ void wlmtk_root_add_workspace(
         wlmtk_workspace_element(workspace_ptr));
 
     // Keep the curtain on top.
-    wlmtk_container_remove_element(
-        &root_ptr->container,
-        wlmtk_rectangle_element(root_ptr->curtain_rectangle_ptr));
-    wlmtk_container_add_element(
+    wlmtk_container_raise_element_to_top(
         &root_ptr->container,
         wlmtk_rectangle_element(root_ptr->curtain_rectangle_ptr));
 
@@ -274,10 +298,11 @@ void wlmtk_root_add_workspace(
     wlmtk_workspace_set_details(
         workspace_ptr, bs_dllist_size(&root_ptr->workspaces));
     wlmtk_workspace_set_root(workspace_ptr, root_ptr);
-    wlmtk_workspace_set_extents(workspace_ptr, &root_ptr->extents);
 
     if (NULL == root_ptr->current_workspace_ptr) {
         _wlmtk_root_switch_to_workspace(root_ptr, workspace_ptr);
+    } else {
+        wl_signal_emit(&root_ptr->events.workspace_changed, workspace_ptr);
     }
 }
 
@@ -297,17 +322,19 @@ void wlmtk_root_remove_workspace(
     wlmtk_element_set_visible(
         wlmtk_workspace_element(workspace_ptr), false);
 
-    if (root_ptr->current_workspace_ptr == workspace_ptr) {
-        _wlmtk_root_switch_to_workspace(
-            root_ptr,
-            wlmtk_workspace_from_dlnode(root_ptr->workspaces.head_ptr));
-    }
-
-    int index = 0;
+    int index = 1;
     bs_dllist_for_each(
         &root_ptr->workspaces,
         _wlmtk_root_enumerate_workspaces,
         &index);
+
+    if (root_ptr->current_workspace_ptr == workspace_ptr) {
+        _wlmtk_root_switch_to_workspace(
+            root_ptr,
+            wlmtk_workspace_from_dlnode(root_ptr->workspaces.head_ptr));
+    } else {
+        wl_signal_emit(&root_ptr->events.workspace_changed, NULL);
+    }
 }
 
 /* ------------------------------------------------------------------------- */
@@ -316,6 +343,22 @@ wlmtk_workspace_t *wlmtk_root_get_current_workspace(wlmtk_root_t *root_ptr)
     return root_ptr->current_workspace_ptr;
 }
 
+/* ------------------------------------------------------------------------- */
+void wlmtk_root_destroy_last_workspace(wlmtk_root_t *root_ptr)
+{
+    wlmtk_workspace_t *ws_ptr = wlmtk_workspace_from_dlnode(
+        root_ptr->workspaces.tail_ptr);
+
+    // Guard clause: Must have further workspaces, not be current workspace,
+    // and not have windows on that workspace.
+    if (1 >= bs_dllist_size(&root_ptr->workspaces) ||
+        ws_ptr == root_ptr->current_workspace_ptr ||
+        !bs_dllist_empty(wlmtk_workspace_get_windows_dllist(ws_ptr))) return;
+
+    wlmtk_root_remove_workspace(root_ptr, ws_ptr);
+    wlmtk_workspace_destroy(ws_ptr);
+}
+
 /* ------------------------------------------------------------------------- */
 void wlmtk_root_switch_to_next_workspace(wlmtk_root_t *root_ptr)
 {
@@ -361,12 +404,14 @@ void wlmtk_root_for_each_workspace(
 /* ------------------------------------------------------------------------- */
 bool wlmtk_root_lock(
     wlmtk_root_t *root_ptr,
-    wlmtk_lock_t *lock_ptr)
+    wlmtk_element_t *element_ptr)
 {
     if (root_ptr->locked) {
-        bs_log(BS_WARNING, "Root already locked by %p", root_ptr->lock_ptr);
+        bs_log(BS_WARNING, "Root already locked by element %p",
+               root_ptr->lock_element_ptr);
         return false;
     }
+    root_ptr->lock_element_ptr = element_ptr;
 
     wlmtk_workspace_enable(root_ptr->current_workspace_ptr, false);
 
@@ -379,8 +424,7 @@ bool wlmtk_root_lock(
 
     wlmtk_container_add_element(
         &root_ptr->container,
-        wlmtk_lock_element(lock_ptr));
-    root_ptr->lock_ptr = lock_ptr;
+        root_ptr->lock_element_ptr);
 
     root_ptr->locked = true;
     return true;
@@ -389,17 +433,17 @@ bool wlmtk_root_lock(
 /* ------------------------------------------------------------------------- */
 bool wlmtk_root_unlock(
     wlmtk_root_t *root_ptr,
-    wlmtk_lock_t *lock_ptr)
+    wlmtk_element_t *element_ptr)
 {
     // Guard clause: Not locked => nothing to do.
     if (!root_ptr->locked) return false;
-    if (lock_ptr != root_ptr->lock_ptr) {
-        bs_log(BS_ERROR, "Lock held by %p, but attempted to unlock by %p",
-               root_ptr->lock_ptr, lock_ptr);
+    if (element_ptr != root_ptr->lock_element_ptr) {
+        bs_log(BS_ERROR, "Lock held by element %p, attempted to unlock by %p",
+               root_ptr->lock_element_ptr, element_ptr);
         return false;
     }
 
-    wlmtk_root_lock_unreference(root_ptr, lock_ptr);
+    wlmtk_root_lock_unreference(root_ptr, element_ptr);
     root_ptr->locked = false;
 
     wlmtk_element_set_visible(
@@ -413,24 +457,22 @@ bool wlmtk_root_unlock(
 }
 
 /* ------------------------------------------------------------------------- */
-void wlmtk_root_lock_unreference(
-    wlmtk_root_t *root_ptr,
-    wlmtk_lock_t *lock_ptr)
+bool wlmtk_root_locked(wlmtk_root_t *root_ptr)
 {
-    if (lock_ptr != root_ptr->lock_ptr) return;
-
-    wlmtk_container_remove_element(
-        &root_ptr->container,
-        wlmtk_lock_element(root_ptr->lock_ptr));
-    root_ptr->lock_ptr = NULL;
+    return root_ptr->locked;
 }
 
 /* ------------------------------------------------------------------------- */
-void wlmtk_root_set_lock_surface(
-    __UNUSED__ wlmtk_root_t *root_ptr,
-    wlmtk_surface_t *surface_ptr)
+void wlmtk_root_lock_unreference(
+    wlmtk_root_t *root_ptr,
+    wlmtk_element_t *element_ptr)
 {
-    wlmtk_surface_set_activated(surface_ptr, true);
+    if (element_ptr != root_ptr->lock_element_ptr) return;
+
+    wlmtk_container_remove_element(
+        &root_ptr->container,
+        root_ptr->lock_element_ptr);
+    root_ptr->lock_element_ptr = NULL;
 }
 
 /* ------------------------------------------------------------------------- */
@@ -459,6 +501,8 @@ void _wlmtk_root_switch_to_workspace(
     } else {
         BS_ASSERT(root_ptr = wlmtk_workspace_get_root(workspace_ptr));
 
+        wlmtk_element_set_visible(
+            wlmtk_workspace_element(workspace_ptr), true);
         if (NULL != root_ptr->current_workspace_ptr) {
             wlmtk_element_set_visible(
                 wlmtk_workspace_element(root_ptr->current_workspace_ptr),
@@ -466,8 +510,6 @@ void _wlmtk_root_switch_to_workspace(
             wlmtk_workspace_enable(root_ptr->current_workspace_ptr, false);
         }
         root_ptr->current_workspace_ptr = workspace_ptr;
-        wlmtk_element_set_visible(
-            wlmtk_workspace_element(root_ptr->current_workspace_ptr), true);
         wlmtk_workspace_enable(root_ptr->current_workspace_ptr, true);
     }
 
@@ -476,21 +518,6 @@ void _wlmtk_root_switch_to_workspace(
         root_ptr->current_workspace_ptr);
 }
 
-/* ------------------------------------------------------------------------- */
-/**
- * Callback for `bs_dllist_for_each` to set extents of the workspace.
- *
- * @param dlnode_ptr
- * @param ud_ptr
- */
-void _wlmtk_root_set_workspace_extents(
-    bs_dllist_node_t *dlnode_ptr,
-    void *ud_ptr)
-{
-    wlmtk_workspace_set_extents(
-        wlmtk_workspace_from_dlnode(dlnode_ptr), ud_ptr);
-}
-
 /* ------------------------------------------------------------------------- */
 /** Callback for bs_dllist_for_each: Destroys the workspace. */
 void _wlmtk_root_destroy_workspace(bs_dllist_node_t *dlnode_ptr, void *ud_ptr)
@@ -513,43 +540,6 @@ void _wlmtk_root_enumerate_workspaces(
     *index_ptr += 1;
 }
 
-/* ------------------------------------------------------------------------- */
-/**
- * Implements @ref wlmtk_element_vmt_t::pointer_motion. Handle pointer moves.
- *
- * When locked, the root container will forward the events strictly only to
- * the lock container.
- *
- * @param element_ptr
- * @param x
- * @param y
- * @param time_msec
- *
- * @return Whether the move was accepted.
- */
-bool _wlmtk_root_element_pointer_motion(
-    wlmtk_element_t *element_ptr,
-    double x, double y,
-    uint32_t time_msec)
-{
-    wlmtk_root_t *root_ptr = BS_CONTAINER_OF(
-        element_ptr, wlmtk_root_t, container.super_element);
-
-    if (!root_ptr->locked) {
-        // TODO(kaeser@gubbe.ch): We'll want to pass this on to the non-curtain
-        // elements only.
-        return root_ptr->orig_super_element_vmt.pointer_motion(
-            element_ptr, x, y, time_msec);
-    } else if (NULL != root_ptr->lock_ptr) {
-        return wlmtk_element_pointer_motion(
-            wlmtk_lock_element(root_ptr->lock_ptr),
-            x, y, time_msec);
-    }
-
-    // Fall-through.
-    return false;
-}
-
 /* ------------------------------------------------------------------------- */
 /**
  * Implements @ref wlmtk_element_vmt_t::pointer_button. Handle button events.
@@ -574,9 +564,9 @@ bool _wlmtk_root_element_pointer_button(
         // elements only.
         return root_ptr->orig_super_element_vmt.pointer_button(
             element_ptr, button_event_ptr);
-    } else if (NULL != root_ptr->lock_ptr) {
+    } else if (NULL != root_ptr->lock_element_ptr) {
         return wlmtk_element_pointer_button(
-            wlmtk_lock_element(root_ptr->lock_ptr),
+            root_ptr->lock_element_ptr,
             button_event_ptr);
     }
 
@@ -608,9 +598,9 @@ bool _wlmtk_root_element_pointer_axis(
         // elements only.
         return root_ptr->orig_super_element_vmt.pointer_axis(
             element_ptr, wlr_pointer_axis_event_ptr);
-    } else if (NULL != root_ptr->lock_ptr) {
+    } else if (NULL != root_ptr->lock_element_ptr) {
         return wlmtk_element_pointer_axis(
-            wlmtk_lock_element(root_ptr->lock_ptr),
+            root_ptr->lock_element_ptr,
             wlr_pointer_axis_event_ptr);
     }
 
@@ -627,18 +617,12 @@ bool _wlmtk_root_element_pointer_axis(
  *
  * @param element_ptr
  * @param wlr_keyboard_key_event_ptr
- * @param key_syms
- * @param key_syms_count
- * @param modifiers
  *
  * @return true if the axis event was handled.
  */
 bool _wlmtk_root_element_keyboard_event(
     wlmtk_element_t *element_ptr,
-    struct wlr_keyboard_key_event *wlr_keyboard_key_event_ptr,
-    const xkb_keysym_t *key_syms,
-    size_t key_syms_count,
-    uint32_t modifiers)
+    struct wlr_keyboard_key_event *wlr_keyboard_key_event_ptr)
 {
     wlmtk_root_t *root_ptr = BS_CONTAINER_OF(
         element_ptr, wlmtk_root_t, container.super_element);
@@ -648,13 +632,11 @@ bool _wlmtk_root_element_keyboard_event(
         // elements only.
         return root_ptr->orig_super_element_vmt.keyboard_event(
             element_ptr,
-            wlr_keyboard_key_event_ptr,
-            key_syms, key_syms_count, modifiers);
-    } else if (NULL != root_ptr->lock_ptr) {
+            wlr_keyboard_key_event_ptr);
+    } else if (NULL != root_ptr->lock_element_ptr) {
         return wlmtk_element_keyboard_event(
-            wlmtk_lock_element(root_ptr->lock_ptr),
-            wlr_keyboard_key_event_ptr,
-            key_syms, key_syms_count, modifiers);
+            root_ptr->lock_element_ptr,
+            wlr_keyboard_key_event_ptr);
     }
 
     // Fall-through: Too bad -- the screen is locked, but the lock element
@@ -662,16 +644,46 @@ bool _wlmtk_root_element_keyboard_event(
     return false;
 }
 
+/* ------------------------------------------------------------------------- */
+/**
+ * Handles wlr_output_layout::events::change. Triggers when the output layout
+ * changes, and we use this for updating the curtain and the layout in each
+ * workspace.
+ *
+ * @param listener_ptr
+ * @param data_ptr
+ */
+void _wlmtk_root_handle_output_layout_change(
+    struct wl_listener *listener_ptr,
+    void *data_ptr)
+{
+    wlmtk_root_t *root_ptr = BS_CONTAINER_OF(
+        listener_ptr, wlmtk_root_t, output_layout_change_listener);
+    struct wlr_output_layout *wlr_output_layout_ptr = data_ptr;
+
+    wlr_output_layout_get_box(wlr_output_layout_ptr, NULL, &root_ptr->extents);
+    wlmtk_rectangle_set_size(
+        root_ptr->curtain_rectangle_ptr,
+        root_ptr->extents.width,
+        root_ptr->extents.height);
+    wlmtk_element_set_position(
+        wlmtk_rectangle_element(root_ptr->curtain_rectangle_ptr),
+        root_ptr->extents.x,
+        root_ptr->extents.y);
+}
+
 /* == Unit tests =========================================================== */
 
 static void test_create_destroy(bs_test_t *test_ptr);
 static void test_workspaces(bs_test_t *test_ptr);
 static void test_pointer_button(bs_test_t *test_ptr);
+static void test_pointer_move(bs_test_t *test_ptr);
 
 const bs_test_case_t wlmtk_root_test_cases[] = {
     { 1, "create_destroy", test_create_destroy },
     { 1, "workspaces", test_workspaces },
     { 1, "pointer_button", test_pointer_button },
+    { 1, "pointer_move", test_pointer_move },
     { 0, NULL, NULL }
 };
 
@@ -681,18 +693,20 @@ void test_create_destroy(bs_test_t *test_ptr)
 {
     struct wlr_scene *wlr_scene_ptr = wlr_scene_create();
     BS_TEST_VERIFY_NEQ_OR_RETURN(test_ptr, NULL, wlr_scene_ptr);
-    wlmtk_root_t *root_ptr = wlmtk_root_create(wlr_scene_ptr, NULL);
+    struct wl_display *wl_display_ptr = wl_display_create();
+    BS_TEST_VERIFY_NEQ_OR_RETURN(test_ptr, NULL, wl_display_ptr);
+    struct wlr_output_layout *wlr_output_layout_ptr = wlr_output_layout_create(
+        wl_display_ptr);
+
+    wlmtk_root_t *root_ptr = wlmtk_root_create(
+        wlr_scene_ptr, wlr_output_layout_ptr);
     BS_TEST_VERIFY_NEQ_OR_RETURN(test_ptr, NULL, root_ptr);
 
     BS_TEST_VERIFY_EQ(
         test_ptr, &root_ptr->events, wlmtk_root_events(root_ptr));
 
-    struct wlr_box extents = { .width = 100, .height = 50 };
-    wlmtk_root_set_extents(root_ptr, &extents);
-    BS_TEST_VERIFY_EQ(test_ptr, 100, root_ptr->extents.width);
-    BS_TEST_VERIFY_EQ(test_ptr, 50, root_ptr->extents.height);
-
     wlmtk_root_destroy(root_ptr);
+    wl_display_destroy(wl_display_ptr);
     wlr_scene_node_destroy(&wlr_scene_ptr->tree.node);
 }
 
@@ -703,16 +717,24 @@ void test_workspaces(bs_test_t *test_ptr)
     wlmtk_util_test_listener_t l;
     struct wlr_scene *wlr_scene_ptr = wlr_scene_create();
     BS_TEST_VERIFY_NEQ_OR_RETURN(test_ptr, NULL, wlr_scene_ptr);
-    wlmtk_root_t *root_ptr = wlmtk_root_create(wlr_scene_ptr, NULL);
+    struct wl_display *wl_display_ptr = wl_display_create();
+    BS_TEST_VERIFY_NEQ_OR_RETURN(test_ptr, NULL, wl_display_ptr);
+    struct wlr_output_layout *wlr_output_layout_ptr =
+        wlr_output_layout_create(wl_display_ptr);
+    wlmtk_root_t *root_ptr = wlmtk_root_create(
+        wlr_scene_ptr, wlr_output_layout_ptr);
     BS_TEST_VERIFY_NEQ_OR_RETURN(test_ptr, NULL, root_ptr);
     BS_TEST_VERIFY_EQ(
         test_ptr, NULL, wlmtk_root_get_current_workspace(root_ptr));
 
     wlmtk_util_connect_test_listener(
         &wlmtk_root_events(root_ptr)->workspace_changed, &l);
+    // Empty? A no-op.
+    wlmtk_root_destroy_last_workspace(root_ptr);
 
     static const wlmtk_tile_style_t tstyle = {};
-    wlmtk_workspace_t *ws1_ptr = wlmtk_workspace_create("1", &tstyle, NULL);
+    wlmtk_workspace_t *ws1_ptr = wlmtk_workspace_create(
+        wlr_output_layout_ptr, "1", &tstyle);
     BS_TEST_VERIFY_NEQ_OR_RETURN(test_ptr, NULL, ws1_ptr);
     wlmtk_root_add_workspace(root_ptr, ws1_ptr);
     BS_TEST_VERIFY_EQ(
@@ -722,8 +744,14 @@ void test_workspaces(bs_test_t *test_ptr)
         wlmtk_workspace_element(ws1_ptr)->visible);
     BS_TEST_VERIFY_EQ(test_ptr, ws1_ptr, l.last_data_ptr);
     BS_TEST_VERIFY_EQ(test_ptr, 1, l.calls);
+    wlmtk_util_clear_test_listener(&l);
+    // Will not destroy the last workspace.
+    wlmtk_root_destroy_last_workspace(root_ptr);
+    BS_TEST_VERIFY_EQ(test_ptr, 1, bs_dllist_size(&root_ptr->workspaces));
+    BS_TEST_VERIFY_EQ(test_ptr, 0, l.calls);
 
-    wlmtk_workspace_t *ws2_ptr = wlmtk_workspace_create("2", &tstyle, NULL);
+    wlmtk_workspace_t *ws2_ptr = wlmtk_workspace_create(
+        wlr_output_layout_ptr, "2", &tstyle);
     BS_TEST_VERIFY_NEQ_OR_RETURN(test_ptr, NULL, ws2_ptr);
     wlmtk_root_add_workspace(root_ptr, ws2_ptr);
     BS_TEST_VERIFY_EQ(
@@ -731,29 +759,49 @@ void test_workspaces(bs_test_t *test_ptr)
     BS_TEST_VERIFY_FALSE(
         test_ptr,
         wlmtk_workspace_element(ws2_ptr)->visible);
+    BS_TEST_VERIFY_EQ(test_ptr, 1, l.calls);
+    BS_TEST_VERIFY_EQ(test_ptr, ws2_ptr, l.last_data_ptr);
+    wlmtk_util_clear_test_listener(&l);
 
     wlmtk_root_remove_workspace(root_ptr, ws1_ptr);
+    BS_TEST_VERIFY_EQ(test_ptr, 1, l.calls);
+    wlmtk_util_clear_test_listener(&l);
     BS_TEST_VERIFY_FALSE(
         test_ptr,
         wlmtk_workspace_element(ws1_ptr)->visible);
-    wlmtk_workspace_destroy(ws1_ptr);
     BS_TEST_VERIFY_EQ(
         test_ptr, ws2_ptr, wlmtk_root_get_current_workspace(root_ptr));
     BS_TEST_VERIFY_TRUE(
         test_ptr,
         wlmtk_workspace_element(ws2_ptr)->visible);
-    BS_TEST_VERIFY_EQ(test_ptr, ws2_ptr, l.last_data_ptr);
-    BS_TEST_VERIFY_EQ(test_ptr, 2, l.calls);
+    // Again: not destroying the workspace.
+    wlmtk_root_destroy_last_workspace(root_ptr);
+    BS_TEST_VERIFY_EQ(test_ptr, 1, bs_dllist_size(&root_ptr->workspaces));
+    BS_TEST_VERIFY_EQ(test_ptr, 0, l.calls);
+
+    // Now add ws1 again. Deleting last workspace
+    wlmtk_root_add_workspace(root_ptr, ws1_ptr);
+    BS_TEST_VERIFY_EQ(test_ptr, 1, l.calls);
+    wlmtk_util_clear_test_listener(&l);
+    BS_TEST_VERIFY_EQ(test_ptr, 2, bs_dllist_size(&root_ptr->workspaces));
+    wlmtk_root_destroy_last_workspace(root_ptr);
+    BS_TEST_VERIFY_EQ(test_ptr, 1, bs_dllist_size(&root_ptr->workspaces));
+    BS_TEST_VERIFY_EQ(test_ptr, 1, l.calls);
+    wlmtk_util_clear_test_listener(&l);
 
     wlmtk_root_remove_workspace(root_ptr, ws2_ptr);
+    BS_TEST_VERIFY_EQ(test_ptr, 1, l.calls);
+    wlmtk_util_clear_test_listener(&l);
     wlmtk_workspace_destroy(ws2_ptr);
     BS_TEST_VERIFY_EQ(
         test_ptr, NULL, wlmtk_root_get_current_workspace(root_ptr));
     BS_TEST_VERIFY_EQ(test_ptr, NULL, l.last_data_ptr);
-    BS_TEST_VERIFY_EQ(test_ptr, 3, l.calls);
+    BS_TEST_VERIFY_EQ(test_ptr, 0, l.calls);
 
     wlmtk_util_disconnect_test_listener(&l);
     wlmtk_root_destroy(root_ptr);
+    wlr_output_layout_destroy(wlr_output_layout_ptr);
+    wl_display_destroy(wl_display_ptr);
     wlr_scene_node_destroy(&wlr_scene_ptr->tree.node);
 }
 
@@ -767,31 +815,32 @@ void test_pointer_button(bs_test_t *test_ptr)
 
     struct wlr_scene *wlr_scene_ptr = wlr_scene_create();
     BS_TEST_VERIFY_NEQ_OR_RETURN(test_ptr, NULL, wlr_scene_ptr);
-    wlmtk_root_t *root_ptr = wlmtk_root_create(wlr_scene_ptr, NULL);
+    struct wl_display *wl_display_ptr = wl_display_create();
+    BS_TEST_VERIFY_NEQ_OR_RETURN(test_ptr, NULL, wl_display_ptr);
+    struct wlr_output_layout *wlr_output_layout_ptr = wlr_output_layout_create(
+        wl_display_ptr);
+    wlmtk_root_t *root_ptr = wlmtk_root_create(
+        wlr_scene_ptr, wlr_output_layout_ptr);
     BS_TEST_VERIFY_NEQ_OR_RETURN(test_ptr, NULL, root_ptr);
     wlmtk_container_add_element(
         &root_ptr->container, &fake_element_ptr->element);
 
     BS_TEST_VERIFY_TRUE(
         test_ptr,
-        wlmtk_root_pointer_motion(root_ptr, 0, 0, 1234));
+        wlmtk_root_pointer_motion(root_ptr, 0, 0, 0, NULL));
     BS_TEST_VERIFY_TRUE(
         test_ptr,
-        fake_element_ptr->pointer_motion_called);
+        fake_element_ptr->pointer_accepts_motion_called);
 
     // Verify that a button down event is passed.
     struct wlr_pointer_button_event wlr_pointer_button_event = {
         .button = 42,
-#if WLR_VERSION_NUM >= (18 << 8)
         .state = WL_POINTER_BUTTON_STATE_PRESSED,
-#else // WLR_VERSION_NUM >= (18 << 8)
-        .state = WLR_BUTTON_PRESSED,
-#endif // WLR_VERSION_NUM >= (18 << 8)
         .time_msec = 4321,
     };
     BS_TEST_VERIFY_TRUE(
         test_ptr,
-        wlmtk_root_pointer_button(root_ptr, &wlr_pointer_button_event));
+        wlmtk_root_pointer_button(root_ptr, &wlr_pointer_button_event, 0));
     wlmtk_button_event_t expected_event = {
         .button = 42,
         .type = WLMTK_BUTTON_DOWN,
@@ -804,14 +853,10 @@ void test_pointer_button(bs_test_t *test_ptr)
         sizeof(wlmtk_button_event_t));
 
     // The button up event should trigger a click.
-#if WLR_VERSION_NUM >= (18 << 8)
     wlr_pointer_button_event.state = WL_POINTER_BUTTON_STATE_RELEASED;
-#else  // WLR_VERSION_NUM >= (18 << 8)
-    wlr_pointer_button_event.state = WLR_BUTTON_RELEASED;
-#endif // WLR_VERSION_NUM >= (18 << 8)
     BS_TEST_VERIFY_TRUE(
         test_ptr,
-        wlmtk_root_pointer_button(root_ptr, &wlr_pointer_button_event));
+        wlmtk_root_pointer_button(root_ptr, &wlr_pointer_button_event, 0));
     expected_event.type = WLMTK_BUTTON_CLICK;
     BS_TEST_VERIFY_MEMEQ(
         test_ptr,
@@ -824,6 +869,74 @@ void test_pointer_button(bs_test_t *test_ptr)
     wlmtk_element_destroy(&fake_element_ptr->element);
 
     wlmtk_root_destroy(root_ptr);
+    wl_display_destroy(wl_display_ptr);
+    wlr_scene_node_destroy(&wlr_scene_ptr->tree.node);
+}
+
+/* ------------------------------------------------------------------------- */
+/** Tests pointer moves. Finicky, because ws switch renders them invisible. */
+void test_pointer_move(bs_test_t *test_ptr)
+{
+    struct wlr_scene *wlr_scene_ptr = wlr_scene_create();
+    BS_TEST_VERIFY_NEQ_OR_RETURN(test_ptr, NULL, wlr_scene_ptr);
+    struct wl_display *wl_display_ptr = wl_display_create();
+    BS_TEST_VERIFY_NEQ_OR_RETURN(test_ptr, NULL, wl_display_ptr);
+    struct wlr_output_layout *wlr_output_layout_ptr = wlr_output_layout_create(
+        wl_display_ptr);
+    struct wlr_output output = { .width = 1024, .height = 768, .scale = 1 };
+    wlmtk_test_wlr_output_init(&output);
+    wlr_output_layout_add_auto(wlr_output_layout_ptr, &output);
+    wlmtk_root_t *root_ptr = wlmtk_root_create(
+        wlr_scene_ptr, wlr_output_layout_ptr);
+    BS_TEST_VERIFY_NEQ_OR_RETURN(test_ptr, NULL, root_ptr);
+
+    static const wlmtk_tile_style_t tstyle = {};
+    wlmtk_workspace_t *ws1_ptr = wlmtk_workspace_create(
+        wlr_output_layout_ptr, "1", &tstyle);
+    BS_TEST_VERIFY_NEQ_OR_RETURN(test_ptr, NULL, ws1_ptr);
+    wlmtk_root_add_workspace(root_ptr, ws1_ptr);
+    wlmtk_workspace_t *ws2_ptr = wlmtk_workspace_create(
+        wlr_output_layout_ptr, "2", &tstyle);
+    BS_TEST_VERIFY_NEQ_OR_RETURN(test_ptr, NULL, ws2_ptr);
+    wlmtk_root_add_workspace(root_ptr, ws2_ptr);
+
+    wlmtk_fake_element_t *fe1 = wlmtk_fake_element_create();
+    BS_TEST_VERIFY_NEQ_OR_RETURN(test_ptr, NULL, fe1);
+    wlmtk_fake_element_set_dimensions(fe1, 40, 20);
+    wlmtk_window_t *w1 = wlmtk_test_window_create(&fe1->element);
+    BS_TEST_VERIFY_NEQ_OR_RETURN(test_ptr, NULL, w1);
+    wlmtk_workspace_map_window(ws1_ptr, w1);
+
+    wlmtk_fake_element_t *fe2 = wlmtk_fake_element_create();
+    BS_TEST_VERIFY_NEQ_OR_RETURN(test_ptr, NULL, fe2);
+    wlmtk_fake_element_set_dimensions(fe2, 40, 20);
+    wlmtk_window_t *w2 = wlmtk_test_window_create(&fe2->element);
+    BS_TEST_VERIFY_NEQ_OR_RETURN(test_ptr, NULL, w2);
+    wlmtk_workspace_map_window(ws2_ptr, w2);
+
+    BS_TEST_VERIFY_TRUE(
+        test_ptr,
+        wlmtk_root_pointer_motion(root_ptr, 5, 10, 42, NULL));
+    wlmtk_root_switch_to_next_workspace(root_ptr);
+    BS_TEST_VERIFY_TRUE(
+        test_ptr,
+        wlmtk_root_pointer_motion(root_ptr, 6, 11, 42, NULL));
+
+    wlmtk_root_remove_workspace(root_ptr, ws2_ptr);
+    wlmtk_root_remove_workspace(root_ptr, ws1_ptr);
+
+    wlmtk_workspace_unmap_window(ws2_ptr, w2);
+    wlmtk_window_destroy(w2);
+    wlmtk_element_destroy(&fe2->element);
+    wlmtk_workspace_unmap_window(ws1_ptr, w1);
+    wlmtk_window_destroy(w1);
+    wlmtk_element_destroy(&fe1->element);
+
+    wlmtk_workspace_destroy(ws2_ptr);
+    wlmtk_workspace_destroy(ws1_ptr);
+
+    wlmtk_root_destroy(root_ptr);
+    wl_display_destroy(wl_display_ptr);
     wlr_scene_node_destroy(&wlr_scene_ptr->tree.node);
 }
 
