@@ -23,17 +23,37 @@
 #include <libbase/libbase.h>
 #include <stdbool.h>
 #include <stdlib.h>
+#include <wayland-server-core.h>
 #include <wayland-server-protocol.h>
 #define WLR_USE_UNSTABLE
 #include <wlr/types/wlr_compositor.h>
 #include <wlr/types/wlr_scene.h>
 #include <wlr/types/wlr_xdg_shell.h>
 #include <wlr/util/box.h>
-#include <wlr/version.h>
 #undef WLR_USE_UNSTABLE
 
 /* == Declarations ========================================================= */
 
+/** State of toolkit popup. */
+struct _wlmaker_xdg_popup_t {
+    /** Base element for the popup. Surface is the content. */
+    wlmtk_base_t              base;
+
+    /** Listener for the `reposition` signal of `wlr_xdg_popup::events` */
+    struct wl_listener        reposition_listener;
+    /** Listener for the `destroy` signal of `wlr_xdg_surface::events`. */
+    struct wl_listener        destroy_listener;
+    /** Listener for the `new_popup` signal of `wlr_xdg_surface::events`. */
+    struct wl_listener        new_popup_listener;
+    /** Listener for the `commit` signal of the `wlr_surface`. */
+    struct wl_listener        surface_commit_listener;
+
+    /** Seat. */
+    struct wlr_seat           *wlr_seat_ptr;
+    /** The WLR popup. */
+    struct wlr_xdg_popup      *wlr_xdg_popup_ptr;
+};
+
 static void handle_reposition(
     struct wl_listener *listener_ptr,
     void *data_ptr);
@@ -71,30 +91,33 @@ wlmaker_xdg_popup_t *wlmaker_xdg_popup_create(
     wlmaker_xdg_popup_ptr->wlr_seat_ptr = wlr_seat_ptr;
     wlmaker_xdg_popup_ptr->wlr_xdg_popup_ptr = wlr_xdg_popup_ptr;
 
-    wlmaker_xdg_popup_ptr->surface_ptr = wlmtk_surface_create(
-        wlr_xdg_popup_ptr->base->surface,
-        wlr_seat_ptr);
-    if (NULL == wlmaker_xdg_popup_ptr->surface_ptr) {
+    if (!wlmtk_base_init(&wlmaker_xdg_popup_ptr->base, NULL)) {
         wlmaker_xdg_popup_destroy(wlmaker_xdg_popup_ptr);
         return NULL;
     }
+    wlmtk_element_set_visible(
+        wlmaker_xdg_popup_element(wlmaker_xdg_popup_ptr), true);
 
+    wlmtk_surface_t *surface_ptr = wlmtk_surface_create(
+        wlr_xdg_popup_ptr->base->surface,
+        wlr_seat_ptr);
+    if (NULL == surface_ptr) {
+        wlmaker_xdg_popup_destroy(wlmaker_xdg_popup_ptr);
+        return NULL;
+    }
+    wlmtk_base_set_content_element(
+        &wlmaker_xdg_popup_ptr->base,
+        wlmtk_surface_element(surface_ptr));
     wlmtk_util_connect_listener_signal(
         &wlr_xdg_popup_ptr->base->surface->events.commit,
         &wlmaker_xdg_popup_ptr->surface_commit_listener,
         handle_surface_commit);
 
-    if (!wlmtk_popup_init(
-            &wlmaker_xdg_popup_ptr->super_popup,
-            wlmtk_surface_element(wlmaker_xdg_popup_ptr->surface_ptr))) {
-        wlmaker_xdg_popup_destroy(wlmaker_xdg_popup_ptr);
-        return NULL;
-    }
     wlmtk_element_extend(
-        wlmtk_popup_element(&wlmaker_xdg_popup_ptr->super_popup),
+        wlmtk_base_element(&wlmaker_xdg_popup_ptr->base),
         &_wlmaker_xdg_popup_element_vmt);
     wlmtk_element_set_position(
-        wlmtk_popup_element(&wlmaker_xdg_popup_ptr->super_popup),
+        wlmtk_base_element(&wlmaker_xdg_popup_ptr->base),
         wlr_xdg_popup_ptr->scheduled.geometry.x,
         wlr_xdg_popup_ptr->scheduled.geometry.y);
 
@@ -103,11 +126,7 @@ wlmaker_xdg_popup_t *wlmaker_xdg_popup_create(
         &wlmaker_xdg_popup_ptr->reposition_listener,
         handle_reposition);
     wlmtk_util_connect_listener_signal(
-#if WLR_VERSION_NUM >= (18 << 8)
         &wlr_xdg_popup_ptr->events.destroy,
-#else // WLR_VERSION_NUM >= (18 << 8)
-        &wlr_xdg_popup_ptr->base->events.destroy,
-#endif // WLR_VERSION_NUM >= (18 << 8)
         &wlmaker_xdg_popup_ptr->destroy_listener,
         handle_destroy);
     wlmtk_util_connect_listener_signal(
@@ -115,12 +134,22 @@ wlmaker_xdg_popup_t *wlmaker_xdg_popup_create(
         &wlmaker_xdg_popup_ptr->new_popup_listener,
         handle_new_popup);
 
+    bs_log(BS_INFO, "Created XDG popup %p", wlmaker_xdg_popup_ptr);
     return wlmaker_xdg_popup_ptr;
 }
 
 /* ------------------------------------------------------------------------- */
 void wlmaker_xdg_popup_destroy(wlmaker_xdg_popup_t *wlmaker_xdg_popup_ptr)
 {
+    bs_log(BS_INFO, "Destroying XDG popup %p", wlmaker_xdg_popup_ptr);
+
+    // For XDG popups, we don't keep reference when adding to a container. So
+    // we need to remove from a potential parent container.
+    wlmtk_element_t *e = wlmtk_base_element(&wlmaker_xdg_popup_ptr->base);
+    if (NULL != e->parent_container_ptr) {
+        wlmtk_container_remove_element(e->parent_container_ptr, e);
+    }
+
     wlmtk_util_disconnect_listener(
         &wlmaker_xdg_popup_ptr->new_popup_listener);
     wlmtk_util_disconnect_listener(
@@ -130,15 +159,17 @@ void wlmaker_xdg_popup_destroy(wlmaker_xdg_popup_t *wlmaker_xdg_popup_ptr)
     wlmtk_util_disconnect_listener(
         &wlmaker_xdg_popup_ptr->surface_commit_listener);
 
-    wlmtk_popup_fini(&wlmaker_xdg_popup_ptr->super_popup);
-
-    if (NULL != wlmaker_xdg_popup_ptr->surface_ptr) {
-        wlmtk_surface_destroy(wlmaker_xdg_popup_ptr->surface_ptr);
-        wlmaker_xdg_popup_ptr->surface_ptr = NULL;
-    }
+    wlmtk_base_fini(&wlmaker_xdg_popup_ptr->base);
     free(wlmaker_xdg_popup_ptr);
 }
 
+/* ------------------------------------------------------------------------- */
+wlmtk_element_t *wlmaker_xdg_popup_element(
+    wlmaker_xdg_popup_t *wlmaker_xdg_popup_ptr)
+{
+    return wlmtk_base_element(&wlmaker_xdg_popup_ptr->base);
+}
+
 /* == Local (static) methods =============================================== */
 
 /* ------------------------------------------------------------------------- */
@@ -151,7 +182,7 @@ void handle_reposition(
         listener_ptr, wlmaker_xdg_popup_t, reposition_listener);
 
     wlmtk_element_set_position(
-        wlmtk_popup_element(&wlmaker_xdg_popup_ptr->super_popup),
+        wlmtk_base_element(&wlmaker_xdg_popup_ptr->base),
         wlmaker_xdg_popup_ptr->wlr_xdg_popup_ptr->scheduled.geometry.x,
         wlmaker_xdg_popup_ptr->wlr_xdg_popup_ptr->scheduled.geometry.y);
 }
@@ -189,11 +220,9 @@ void handle_new_popup(
         return;
     }
 
-    wlmtk_element_set_visible(
-        wlmtk_popup_element(&new_popup_ptr->super_popup), true);
-    wlmtk_popup_add_popup(
-        &wlmaker_xdg_popup_ptr->super_popup,
-        &new_popup_ptr->super_popup);
+    wlmtk_base_push_element(
+        &wlmaker_xdg_popup_ptr->base,
+        wlmaker_xdg_popup_element(new_popup_ptr));
 
     bs_log(BS_INFO, "XDG popup %p: New popup %p",
            wlmaker_xdg_popup_ptr, wlr_xdg_popup_ptr);
@@ -226,7 +255,7 @@ void _wlmaker_xdg_popup_element_destroy(
 {
     wlmaker_xdg_popup_t *wlmaker_xdg_popup_ptr = BS_CONTAINER_OF(
         element_ptr, wlmaker_xdg_popup_t,
-        super_popup.super_container.super_element);
+        base.super_container.super_element);
 
     wlmaker_xdg_popup_destroy(wlmaker_xdg_popup_ptr);
 }
